#!/usr/bin/python
#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2015. All Rights Reserved.
#
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************
#############################################################################
# Name: cleanEnvironment.py
#
# this will use 
# -- globalProps.py to contain global variables
# -- processTagsLib.py as a function library
#
# Purpose: 
#    1. clean the UCD environment of all properties that may confuse things if the environment is reused
#       eg {globalTagList} and {DatabaseType}
#    2. delete the env.properties file
#    3. create and empty env.properties file
#
###################################################


####################################################
# ===================== INPUTS =====================
#
#    AGENT_HOME ------- $AGENT_HOME  or  %AGENT_HOME%
#    UCDSERVER -------- $AH_WEB_URL  or  %AH_WEB_URL%
#    AUTH_TOKEN ------- $AUTH_TOKEN  or %AUTH_TOKEN%
#    APPLICATION_ID --- ${p:application.id}
#    ENVIRONMENT_ID---- ${p:environment.id}
#    COMPONENT_ID ----- ${p:resource.id}
#    AGENT_ID --------- ${p:agent.id}
#
####################################################



####################################################
# ===================== FUNCTIONS =====================
####################################################

#########################################
#
# Function processInputs()
#
# expect 7 inputs 
#####################################

def processInputs():

    globalProps.LOCALSERVERPROPERTYFOLDER = "/etc"
    
    INPUTS_LIST = "[AGENT_HOME][UCDSERVER][AUTH_TOKEN][APPLICATION_ID][ENVIRONMENT_ID][COMPONENT_ID][AGENT_ID]"
    # always 1 more than the number of inputs as you add 1 for the script name
    NUMBER_OF_ARGS = 8
    
    if (len(sys.argv) == NUMBER_OF_ARGS):
        globalProps.SCRIPT_NAME = sys.argv[0]
        globalProps.AGENT_HOME = sys.argv[1]
        globalProps.UCDSERVER = sys.argv[2]
        globalProps.AUTH_TOKEN = sys.argv[3]
        globalProps.APPLICATION_ID = sys.argv[4]
        globalProps.ENVIRONMENT_ID = sys.argv[5]
        globalProps.COMPONENT_ID = sys.argv[6]
        globalProps.AGENT_ID = sys.argv[7]

    else:
        print "ERROR: inputs should be " + INPUTS_LIST
        print "NOT [" + str(len(sys.argv)) + "] [" + str(sys.argv) + "]"
        sys.exit(1)

    print "AGENT_HOME: " + globalProps.AGENT_HOME
    print "UCDSERVER: " + globalProps.UCDSERVER
    print "AUTH_TOKEN: " + globalProps.AUTH_TOKEN
    print "APPLICATION_ID: " + globalProps.APPLICATION_ID
    print "ENVIRONMENT_ID: " + globalProps.ENVIRONMENT_ID
    print "COMPONENT_ID: " + globalProps.COMPONENT_ID
    print "AGENT_ID: " + globalProps.AGENT_ID
    



# ===================== main =====================
#
####################################################

import sys

# local file
from globalProps import *
from processTagsLib import *

# initialise global variables
globalProps.init()
globalProps.bDEBUGFLAG = "False"

# initialise local variables
seperatorString = ","


print "====== process inputs ========="
processInputs()

print "\n====== determine operating system ========="
determineOS()

print "\n====== get environment properties ========="
setEnvProps()

print "\n====== check the UCD agent <--> server connection ========="
testAgentConnection()


## clean UCD environment
##################################
print "\n====== cleanUCDEnvironmentProperty using the ucdclient api ======"
setUCDEnvironmentProperty("globalTagListing", "")
# setUCDEnvironmentProperty("DatabaseType", "")


## delete the local server environment
######################################
print "\n====== delete local server property file ======"
deleteFileIfExists(globalProps.LOCALSERVERPROPERTYFILE)


## create empty local server environment
######################################
print "\n====== create local server property file ======"
createFile(globalProps.LOCALSERVERPROPERTYFILE)


print "\n=== EOF " + globalProps.SCRIPT_NAME + "==="
sys.exit(0)
