#!/usr/bin/python
#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2015. All Rights Reserved.
#
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************
#############################################################################
# Name: processTags.py
#
# this will use 
# -- globalProps.py to contain global variables
# -- processTagsLib.py as a function library
#
# Purpose: 
#    1. extract the TAGS from a "server",
#       create a new element [hostname]:[csv list of tags]
#       create or add to UCD environment property {globalTagList} and populate with the new element
#    2. create localTagList and localHostname
#       add or update their entries in the env.properties file
#
###################################################


####################################################
# ===================== INPUTS =====================
#
#    AGENT_HOME ------- $AGENT_HOME  or  %AGENT_HOME%
#    UCDSERVER -------- $AH_WEB_URL  or  %AH_WEB_URL%
#    AUTH_TOKEN ------- $AUTH_TOKEN  or %AUTH_TOKEN%
#    APPLICATION_ID --- ${p:application.id}
#    ENVIRONMENT_ID---- ${p:environment.id}
#    COMPONENT_ID ----- ${p:resource.id}
#    AGENT_ID --------- ${p:agent.id}
#
####################################################



####################################################
# ===================== FUNCTIONS =====================
####################################################

#########################################
#
# Function processInputs()
#
# expect 7 inputs 
#####################################

def processInputs():

    globalProps.LOCALSERVERPROPERTYFOLDER = "/etc"
    
    INPUTS_LIST = "[AGENT_HOME][UCDSERVER][AUTH_TOKEN][APPLICATION_ID][ENVIRONMENT_ID][COMPONENT_ID][AGENT_ID]"
    # always 1 more than the number of inputs as you add 1 for the script name
    NUMBER_OF_ARGS = 8
    
    if (len(sys.argv) == NUMBER_OF_ARGS):
        globalProps.SCRIPT_NAME = sys.argv[0]
        globalProps.SCRIPT_FOLDER = os.path.dirname(os.path.realpath(globalProps.SCRIPT_NAME))
        globalProps.AGENT_HOME = sys.argv[1]
        globalProps.UCDSERVER = sys.argv[2]
        globalProps.AUTH_TOKEN = sys.argv[3]
        globalProps.APPLICATION_ID = sys.argv[4]
        globalProps.ENVIRONMENT_ID = sys.argv[5]
        globalProps.COMPONENT_ID = sys.argv[6]
        globalProps.AGENT_ID = sys.argv[7]

    else:
        print "ERROR: inputs should be " + INPUTS_LIST
        print "NOT [" + str(len(sys.argv)) + "] [" + str(sys.argv) + "]"
        sys.exit(1)

    print "AGENT_HOME: " + globalProps.AGENT_HOME
    print "UCDSERVER: " + globalProps.UCDSERVER
    print "AUTH_TOKEN: " + globalProps.AUTH_TOKEN
    print "APPLICATION_ID: " + globalProps.APPLICATION_ID
    print "ENVIRONMENT_ID: " + globalProps.ENVIRONMENT_ID
    print "COMPONENT_ID: " + globalProps.COMPONENT_ID
    print "AGENT_ID: " + globalProps.AGENT_ID
    



# ===================== main =====================
#
####################################################

import sys

# local file
from globalProps import *
from processTagsLib import *

# initialise global variables
globalProps.init()
globalProps.bDEBUGFLAG = "False"

# initialise local variables
seperatorString = ","


print "====== process inputs ========="
processInputs()

print "\n====== determine operating system ========="
determineOS()

print "\n====== get environment properties ========="
setEnvProps()

print "\n====== check the UCD agent <--> server connection ========="
testAgentConnection()

print "\n====== getServerResourceParentTags using the ucdclient api ======"
resourceTags = getServerResourceParentTags(globalProps.COMPONENT_ID)

print "\n====== checkTagRules ======"
checkTagRules(resourceTags, seperatorString)

print "\n====== getAppsFromTags ======"
appString = getAppsFromTags(resourceTags, seperatorString)

#print "\n====== getDatabaseFromTags ======"
#databaseType = getDatabaseFromTags(resourceTags, seperatorString)

print "\n====== getWasRoleFromTags ======"
wasRole = getWasRoleFromTags(resourceTags, seperatorString)


print "\n====== get Server Hostname ======"
hostname = getHostname()
if "Property not found" == hostname:
    print "ERROR: agent property [Hostname] not found"
    sys.exit(1)

if "True" == checkForUpperCaseChar(hostname):
    print "ERROR: hostname [" + hostname + "] contains uppercase characters"
    sys.exit(1)
else:
    print "hostname [" + hostname + "] DOES NOT contains uppercase characters"



## update UCD environment
##################################
print "\n====== updateUCDEnvironmentProperty using the ucdclient api ======"
updateUCDEnvironmentProperty("globalTagListing", hostname, resourceTags)

# if  "noDatabaseFound" != databaseType:
#     setUCDEnvironmentProperty("DatabaseType", databaseType)


## update local server environment
#######################################
print "\n====== update local server property file (localHostname=..) ======"
updateServerPropertiesFile(globalProps.LOCALSERVERPROPERTYFILE, "localHostname", hostname)

print "\n====== update local server property file (localTags=..) ======\n"
updateServerPropertiesFile(globalProps.LOCALSERVERPROPERTYFILE, "localTags", resourceTags)

print "\n====== update local server property file (APPS=..) ======"
updateServerPropertiesFile(globalProps.LOCALSERVERPROPERTYFILE, "APPS", appString)

if  "noWasFound" != wasRole:
    print "\n====== update local server property file (wasRole=..) ======"
    updateServerPropertiesFile(globalProps.LOCALSERVERPROPERTYFILE, "WASRole", wasRole)

#if  "noDatabaseFound" != databaseType:
#    print "\n====== update local server property file (DATABASE_TYPE=..) ======"
#    updateServerPropertiesFile(globalProps.LOCALSERVERPROPERTYFILE, "DATABASE_TYPE", databaseType)


## check Custom LDAP properties exist
#######################################
if  "was" == wasRole:
    print "\n====== checking file [Custom_LDAPSecurity.properties] exists on WAS server ======"
    checkCustomLdapFileExists()



print "\n=== EOF " + globalProps.SCRIPT_NAME + "==="
sys.exit(0)
