#!/bin/bash
#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2015. All Rights Reserved.
#
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************

########################################################
#
# Script : doEnc.sh
# Purpose : base64 encode input and write/update key:value to file
#
# $1 = propertyName
# $2 = propertyValue
# $3 = file to update
# $4 = (bool) encode or not
#
#########################################################

scriptName=$0
numberOfInputs=$#
allInputs=$*

##############
## 
## FUNCTIONS
##
###############

## usage exitWithErrMsg "tempString"
exitWithErrMsg()
{
  tempString=$1
  echo "-- ERROR [${scriptName}] --"
  echo "-- message: $tempString"
  exit 1
}


## usage errorWrongNumberOfInputs
errorWrongNumberOfInputs()
{
  echo "ERROR: wrong number of input parameters, expected [$expectedInputParameterCount] not [$numberOfInputs]"
  echo "from command [$0] [$allInputs]" 
  echo " "
  echo "Usage: [propertyName] [propertyValue] [file to update] [(bool) encode or not]"
  exitWithErrMsg "wrong number of input parameters"
}


## usage checkBoolean bFlag
## -- accepts true, false, TRUE, FALSE (and a mismatch of case!)
checkBoolean()
{

  tempFlag=$1
  # convert to lowercase
  convertedFlag=`echo $tempFlag | tr [A-Z] [a-z]`
  
  if [ "true" == ${convertedFlag} ] || [ "false" == ${convertedFlag} ];then
    echo "[${tempFlag}] is boolean"
  else
    errMsg="[${tempFlag}] is not boolean"
    exitWithErrMsg "${errMsg}"
  fi

}

# update property file with the new key:value pair
updateServerPropertiesFile()
{
  origFile=$1
  tempKey=$2
  newValue=$3
  encFlag=$4

  tempFile=${origFile}.tmp
  ls -la $origFile

  matchKey=${tempKey}"="

  # do not write encoded value to console / log
  if [ "true" == ${encFlag} ]; then 
    tempValueString="****"
  else
    tempValueString=${newValue}
  fi

  # does key already exist?
  cat ${origFile} | grep ${matchKey} > /dev/NUL
  if [ $? -gt 0 ]; then
    # key does not exist, so add it to the file
    echo "add new key ["${tempKey}"] ["${tempValueString}"]"
    echo ${tempKey}"="${newValue}>> ${origFile}
  else
    # key does exist, so update it
    echo "update existing key with ["${tempKey}"] ["${tempValueString}"]"
    # lines starting with # should not be updated
    sed "s/^ *${tempKey}=.*/${tempKey}=$newValue/g" ${origFile} > ${tempFile}
    mv ${tempFile} ${origFile}
  fi

  chmod 600 $origFile
  ls -la $origFile
}



#############
## MAIN
#############
expectedInputParameterCount=4
propertyName=$1
propertyValue=$2
envFile=$3
bEncode=$4


# check inputs
if [ $expectedInputParameterCount -ne $numberOfInputs ]; then
  errorWrongNumberOfInputs
fi

checkBoolean $bEncode
# convert to lowercase for later use
bEncode="$(echo $bEncode | tr '[A-Z]' '[a-z]')"

if [ ! -f $envFile ]; then
  echo "creating file [$envFile]"
  touch $envFile
fi

# encode value if required
if [ "true" == ${bEncode} ]; then 
  encVal=`echo $propertyValue | base64`
else
  encVal=$propertyValue
fi

# update the property file
updateServerPropertiesFile $envFile $propertyName "$encVal" $bEncode


################
## === finished ====
echo "-- EOF [${scriptName}] SUCCESS --"
exit 0