#!/bin/sh
#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2015. All Rights Reserved.
#
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************
. /etc/env.properties

# get credentials
userCreds=$AUTO_PREP/UserParameters/userCreds.properties
. $userCreds

# from the environment
WAS_SCRIPTS_DIR=${WAS_COMMON_DIR}/WASScripts
WAS_HOME=$WAS_PROFILE_ROOT
echo "WAS_SCRIPTS_DIR=$WAS_SCRIPTS_DIR"
echo "WAS_PROFILE_ROOT=$WAS_PROFILE_ROOT"
echo "WAS_HOME=$WAS_HOME"
echo "CONFIGURE_APP_SERVER_SCRIPTS_DIR=$CONFIGURE_APP_SERVER_SCRIPTS_DIR"
echo "COMPONENT=$COMPONENT"

echo "----------------------- select apps for war deployment command -------------------"
# applicaton to be installed may be release dependant
apps=$APPS
apps=${apps//,/ }
echo "apps ($apps)"

# if WAS_USER_NAME doesn't exist, then WASCommon wasn't run
if [ -z "$WAS_USER_NAME" ]; then 
   echo "ERROR: WAS_USER_NAME variable did not set"
   exit 1
fi
echo "WAS_USER_NAME=$WAS_USER_NAME"

cd $WAS_HOME/bin

for APP in $apps
do
    echo APP is [$APP]
	# ----
	if [ "lqe" == $APP ]; then
  	# update JVM properties
 	 # ----
	  timestamp=`date +%s`
	  echo "DEBUG (IA): current epoch time is ["$timestamp"] Sec"
 	 # ----

 	 echo "Add LQE specific JVM properties to WAS appserver....."
 	 su - $OSUserName -c "$WAS_HOME/bin/wsadmin.sh -lang jython -f $CONFIGURE_APP_SERVER_SCRIPTS_DIR/UpdateJvmLQE.jy $CONFIGURE_APP_SERVER_SCRIPTS_DIR"
 	 if [ $? -ne 0 ]
 	 then
 	   echo "LQE specific JVM properties update failed"
 	   exit 1
 	 fi
  
 	 timestampDone=`date +%s`
 	 timeTaken=`expr $timestampDone - $timestamp`
	  echo "DEBUG (IA): Add LQE specific JVM properties to WAS appserver took ["$timeTaken"] Sec"
  
	  echo "LQE specific JVM properties update succeeded"
 	 echo "-----------------------"
 	 echo " "
	fi
	


	#install apps on WAS
	# ----
	timestamp=`date +%s`
	echo "DEBUG (IA): current epoch time is ["$timestamp"] Sec"
	# ----
	echo "Install Application ($APP) to WAS appserver....."
	su - $OSUserName -c "$WAS_HOME/bin/wsadmin.sh -lang jython -f $CONFIGURE_APP_SERVER_SCRIPTS_DIR/InstallApps.jy ${APP} ${WAS_COMMON_DIR}"
	if [ $? -ne 0 ]
	then
		timestampDone=`date +%s`
		timeTaken=`expr $timestampDone - $timestamp`
		echo "DEBUG (IA): Install Application ($APP) to WAS appserver took ["$timeTaken"] Sec"
  		echo "Application [${APP}] Deployment failed"
		exit 1
	fi

	timestampDone=`date +%s`
	timeTaken=`expr $timestampDone - $timestamp`
	echo "DEBUG (IA): Install Application ($APP) to WAS appserver took ["$timeTaken"] Sec"

	echo "Application [${APP}] deployment succeeded"
	echo "-----------------------"
	echo " "
done

#If WASCommon installed
if [ -e "$WAS_SCRIPTS_DIR" ]; then
  # ----
  timestamp=`date +%s`
  echo "DEBUG (IA): current epoch time is ["$timestamp"] Sec"
  # ----
  echo "------Restart WebSphere Server------"
  $WAS_SCRIPTS_DIR/RestartSingleWAS.sh
  if [ $? -ne 0 ]; then
    exit 1
  fi
  
  timestampDone=`date +%s`
  timeTaken=`expr $timestampDone - $timestamp`
  echo "DEBUG (IA): Restart WebSphere Server took ["$timeTaken"] Sec"
  
  echo "Apps deployed and WebSphere Server restarted"
  # ----
  timestamp=`date +%s`
  echo "DEBUG (IA): current epoch time is ["$timestamp"] Sec"
  # ----
  exit 0
fi



echo "------Restart (Stop) WebSphere Server------"
# ----
timestamp=`date +%s`
echo "DEBUG (IA): current epoch time is ["$timestamp"] Sec"
# ----
su - $OSUserName -c "$WAS_HOME/bin/stopServer.sh server1"
if [ $? -ne 0 ]
then
  echo "WebSphere Server failed to stop"
  exit 1
fi

timestampDone=`date +%s`
timeTaken=`expr $timestampDone - $timestamp`
echo "DEBUG (IA): Restart (Stop) WebSphere Server took ["$timeTaken"] Sec"

echo "WebSphere Server stopped"
echo "-----------------------"
echo " "

echo "------Restart (Start) WebSphere Server------"
# ----
timestamp=`date +%s`
echo "DEBUG (IA): current epoch time is ["$timestamp"] Sec"
# ----
su - $OSUserName -c "$WAS_HOME/bin/startServer.sh server1"
if [ $? -ne 0 ]
then
  echo "WebSphere Server failed to start"
  exit 1
fi

timestampDone=`date +%s`
timeTaken=`expr $timestampDone - $timestamp`
echo "DEBUG (IA): Restart (Start) WebSphere Server took ["$timeTaken"] Sec"

# ----
timestamp=`date +%s`
echo "DEBUG (IA): current epoch time is ["$timestamp"] Sec"
# ----
echo "Apps deployed and WebSphere Server restarted"
exit 0

