#!/bin/bash

#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2015. All Rights Reserved.
#
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************


#######################
# ---- Functions ---- #
#######################

## usage copyLPTAKey() Source Destination
copyLPTAKey()
{
   source=$1
   destination=$2
   if [ $source == local ]
   then
      echo "copying LPTA Key from IHS node to ["$destination"]...."
      scp /tmp/jtsLTPAkey $OSUserName@$destination:/tmp/jtsLTPAkey
      if [ $? -gt 0 ]; then
        echo "ERROR: failed to LPTA key"
        exit 1
      fi 
   else
      echo "copying LPTA Key ["$source"] to IHS node...."
      scp $OSUserName@$source:/tmp/jtsLTPAkey /tmp/jtsLTPAkey
      if [ $? -gt 0 ]; then
        echo "ERROR: failed to LPTA key"
        exit 1
      fi
   fi
}


## usage copyCertificateByNumber() appName serverHost
copyCertificateByNumber()
{
    serverHost=$1
    number=$2
    echo "copying server ["${number}"] ["${serverHost}"] root certificate..."
    scp $OSUserName@$serverHost:/tmp/serverroot.cer /tmp/serverroot_$number.cer
	if [ ! -f /tmp/serverroot_$number.cer ]; then
       echo "ERROR: failed to copy root certificate from [${serverHost}]"
       exit 1
    fi   
}


## usage copyDwaCertificate() serverHost
copyDwaCertificate()
{
    serverHost=$1
    echo "copying dwa server ["${serverHost}"] root certificate..."
    scp $OSUserName@$serverHost:/tmp/serverroot_dwa.cer /tmp/serverroot_dwa.cer
    if [ $? -gt 0 ]; then
       echo "ERROR: failed to copy dwa root certificate"
       exit 1
    fi 
}


## usage setupSSH() appName serverHost
setupSSH()
{
    appName=$1
    serverHost=$2
    HOME=/root
    echo "Setup SSH for [$appName], password is [**XX**]..."
    HOME=$HOME $IHS_Config_DIR/sshScripts/setupSSH -p $tempOSPassword $OSUserName@$serverHost
	if [ $? != 0 ]; then
        echo "ERROR: Failed to set up ssh for  $OSUserName@$serverHost"
        exit 1
    fi
	
}


##################################################################################
#################
# ---- MAIN --- #
#################

scriptName=$0
##if [ $# == 0 ]; then
##  echo "ERROR: script "${scriptName}" has no input parameters"
##  exit 1
##fi

envProp=/etc/env.properties
touch $envProp
. $envProp

# get credentials
userCreds=$AUTO_PREP/UserParameters/userCreds.properties
. $userCreds

# command to change file mode for OSUser
sshChmodRead="sudo chmod o+r $userCreds 2>&1"
sshChmodRevert="sudo chmod 600 $userCreds 2>&1"

# Get IHSServerKeyStorePassword for the setup
# ======================================
tempIHSServerKeyStorePassword=`echo ${IHSServerKeyStorePassword} | base64 -d`
tempOSPassword=`echo ${OSUserPassword} | base64 -d`

echo "OSUserName: [$OSUserName]"
echo "OSUserPassword: [**XX**]"
echo "OSUserGroup: [$OSUserGroup]"

IHS_Config_DIR=`pwd -P`
echo "" >> $envProp
echo IHS_Config_DIR=$IHS_Config_DIR >> $envProp
echo IHS_Config_DIR: [$IHS_Config_DIR]


##globalTagListing=$1
##echo "globalTagListing ["${globalTagListing}"]"


###########################
# ---- processTagList --- #
###########################
##sh ./processGlobalTagList.sh ${globalTagListing}
##if [ $? -gt 0 ]; then
##  echo "ERROR: failed to process the globalTagListing"
##  exit 1
##fi
###reload environment
##. $envProp

#############################
# ---- nodes to process --- #
#############################
echo "NUMBER_OF_CUSTOM_NODES to process ["${NUMBER_OF_CUSTOM_NODES}"]"
echo "NUMBER_OF_DWA_NODES to process ["${NUMBER_OF_DWA_NODES}"]"


##########################
# ---- set up expect --- 
##########################
chmod 755 $IHS_Config_DIR/sshScripts/*
# expect package installed as a system preReq


###########################################
# ---- Parse the script arguments ------- #
# ---- setup remote SSH connection ------ #
# ---- access the remote server for ----- #
# ---- WAS certificate               ---- #
###########################################

# -- process WAS servers
i=1
while [ $i -le $NUMBER_OF_CUSTOM_NODES ]
do
  c_host_var=NODE${i}_HostName
  eval c_host=\$$c_host_var
  
  setupSSH NODE${i} $c_host
  if [ $? == 0 ]
  then
     copyCertificateByNumber $c_host $i
  else
     exit 1
  fi

  # -- Copy jts LTPA key to the IHS node
  app_var=NODE${i}_AppList
  eval apps=\$$app_var
  echo $apps |grep jts
  if [ $? == 0 ]
  then
     copyLPTAKey $c_host "local"
     jtsHostName=$c_host
  fi

  i=$(( i+1 ))
done

# -- copy JTS LTPA key to all other WAS nodes and import the key, the key name is jtsLTPAkey
i=1
while [ $i -le $NUMBER_OF_CUSTOM_NODES ]
do
  c_host_var=NODE${i}_HostName
  eval c_host=\$$c_host_var
  echo "JTSHOSTNAME: [$jtsHostName]"
  echo "c_host: [$c_host]"
  if [ $jtsHostName != $c_host ]
  then
     copyLPTAKey "local" $c_host
     tempWASdir=`ssh $OSUserName@$c_host grep -m 1 WAS_COMMON_DIR /etc/env.properties | cut -d = -f2`
     echo "WASCommon directory name [$tempWASdir]"
     echo "importing the jts LTPA key..."
	 ssh -t -t  -o ConnectTimeout=30 $OSUserName@$c_host "$sshChmodRead" 2> ${OSUserName}_${serverHost}_import_sshChmodRead.log
     ssh -o ConnectTimeout=300 $OSUserName@$c_host "echo "y" | sh ${tempWASdir}/EnableSSO.sh import"
	 if [ $? -gt 0 ]; then
	    echo "ERROR: failed to Import jts LTPA key..."
        exit 1
     fi
     ssh -t -t -o ConnectTimeout=30 $OSUserName@$c_host "$sshChmodRevert" 2> ${OSUserName}_${serverHost}_import_sshChmodRevert.log
  fi
  i=$(( i+1 ))
done

# -- process DWA server
if [ "NO" == "NO"${dwahost} ]; then
  echo "skipping DWA processing"
  dwaPresentFlag=FALSE
else
  dwaPresentFlag=TRUE
  setupSSH "dwa" ${dwahost}
  copyDwaCertificate ${dwahost}
fi


###########################################################
# --- enable SSL and create a self signed certificate --- #
###########################################################
echo "--- Enable SSL from httpd.conf ---"
sh $IHS_Config_DIR/enableSSL.sh
echo "--- create ihsserverkey key store ---"
cd $IHS_ROOT
$IHS_ROOT/bin/gskcmd -keydb -create -db ihsserverkey.kdb -pw $tempIHSServerKeyStorePassword -expire 365 -stash -type cms
if [ $? -gt 0 ]; then
        echo "ERROR: failed to create ihsserver key store"
        exit 1
fi
echo "--- Create self sign certificate ---"
$IHS_ROOT/bin/gskcmd -cert -create -db ihsserverkey.kdb -label ihsserver -expire 365 -dn "CN=$ihshost" -default_cert yes  -pw $tempIHSServerKeyStorePassword
if [ $? -gt 0 ]; then
        echo "ERROR: failed to create self sign certificate"
        exit 1
fi
if [ "TRUE" == $dwaPresentFlag ]; then
  echo "--- add dwa certificate to ihsserverkey key store ---"
  $IHS_ROOT/bin/gskcmd -cert -add -db ihsserverkey.kdb -label DWA -pw $tempIHSServerKeyStorePassword -file /tmp/serverroot_dwa.cer
  if [ $? -gt 0 ]; then
        echo "ERROR: failed to add dwa certificate to ihsserver keystore"
        exit 1
   fi
fi

echo "--- list certificates in the ihsserverkey key store ---"
$IHS_ROOT/bin/gskcmd -cert -list -db ihsserverkey.kdb -pw $tempIHSServerKeyStorePassword 


##################################################
# --- add server root certificate to plugin, --- #
# ---         using SSL hand shake           --- #
##################################################
sh $IHS_Config_DIR/AddCertificates.sh


#######################################################################
# --- Merge Plugin-cfg.xml and put it in webservermerge directory --- #
#######################################################################
echo "---- make webservermerge log directory ---"
mkdir ${Plugins_ROOT}/logs/webservermerge
echo "--- copy webserver_template directory to merge directory ---"
cp -fr $IHS_Config_DIR/webserver_template ${Plugins_ROOT}/config/webservermerge 
#cp -f /opt/IBM/Plugins/config/webserver/* ${Plugins_ROOT}/config/webservermerge 
echo "--- Merge Plugin-cfg.xml and put it on webservermerge directory ---"
sh $IHS_Config_DIR/BuildPluginFile.sh
cp -f $IHS_Config_DIR/plugin-cfg.xml ${Plugins_ROOT}/config/webservermerge/plugin-cfg.xml


#########################################################
# --- Restart webserver and all application servers --- #
#########################################################
echo "--- Restart webserver ---"
sh  $IHS_Config_DIR/RestartWebServer.sh
echo "--- restart all application servers ---"
sh $IHS_Config_DIR/RestartApplicationSrv.sh
if [ $? -gt 0 ]; then
    echo "ERROR: failed to RestartApplicationSrv"
    exit 1
fi

###################
# --- the End --- #
###################

echo "-- EOF [${scriptName}] --"
exit 0