#!/bin/sh
#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2015. All Rights Reserved.
#
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************

#########################################################################
# run this from the IHS server after a successful config IHS step
#########################################################################

. /etc/env.properties

# get credentials
userCreds=$AUTO_PREP/UserParameters/userCreds.properties
. $userCreds


#######################
# ---- Functions ---- #
#######################

getRootUlimits()
{
  serverHost=$1
  
  echo "=== get ulimits for [root] [$serverHost] ==="
  ssh -o ConnectTimeout=30 root@$serverHost "ulimit -a"
  echo ""
}

getOSuserUlimits()
{
  serverHost=$1
  
  echo "=== get ulimits for [$OSUserName] [$serverHost] ==="
  ssh -o ConnectTimeout=30 $OSUserName@$serverHost "ulimit -a"
  echo ""
}

getLimitsForProc1()
{
serverHost=$1
  
  echo "=== get ulimits for [process 1] [$serverHost] ==="
  ssh -o ConnectTimeout=30 root@$serverHost "cat /proc/1/limits"
  echo ""
}


getLimitsForWASprocess()
{
serverHost=$1
  
  echo "=== get ulimits for [WAS process] [$serverHost] ==="
  wasPs=`ssh root@$serverHost ps -ef | grep WebSphere`
  wasId=`echo $wasPs | cut -d" " -f2`
  echo [${wasId}]
  ssh -o ConnectTimeout=30 root@$serverHost "cat /proc/${wasId}/limits"
  echo ""
}


#######################
# ---- MAIN ---- #
#######################


echo "--- getting ulimit info across the WAS servers ---"
echo ""
i=1
while [ $i -le $NUMBER_OF_CUSTOM_NODES ]
do
  c_host_var=NODE${i}_HostName
  eval c_host=\$$c_host_var
  
  app_var=NODE${i}_AppList
  eval apps=\$$app_var
  
  echo "--- WAS server [$c_host] [$apps] ---"
  echo "--------------------------------------------------------------"
  
  getRootUlimits $c_host
  getOSuserUlimits $c_host
  getLimitsForProc1 $c_host
  getLimitsForWASprocess $c_host
  
  i=$(( i+1 ))
done
echo "----------"


exit 0
