#!/bin/sh
#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2015. All Rights Reserved.
#
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************
#############
## FUNCTIONS 
#############

checkOS()
{
	####################################
	# may need to check OS 
	bRH6flag=FALSE
	bRH7flag=FALSE
	
	RHversion=`cat /etc/redhat-release`
	echo "Current Red Hat Release $RHversion"
	
	echo $RHversion | grep "Maipo"
	if [ $? -eq 0 ]; then
	  echo "OS is Red Hat 7"
	  bRH7flag=TRUE
	fi
	echo $RHversion | grep "Santiago"
	if [ $? -eq 0 ]; then
	  echo "OS is Red Hat 6"
	  bRH6flag=TRUE
	fi
}

. /etc/env.properties

envProp=/etc/env.properties
. $envProp

# get credentials
userCreds=$AUTO_PREP/UserParameters/userCreds.properties
. $userCreds

echo "WASCOMMON=$WAS_COMMON_DIR"
echo "WAS_PROFILE_ROOT=$WAS_PROFILE_ROOT"
echo "WAS_USER_NAME=$WAS_USER_NAME"

## -----------------------------------------------------------------------------------------
## ----- the following bit of code should eventually be moved to CoreAutomationPrep.sh -----

# xvfb only required on RM and DM servers
echo "if required [RM & DM] ensure XVFB is running"

# reuse the $checkPreReqXvfbFlag as that will be available in CoreAutomationPrep.sh
checkPreReqXvfbFlag=FALSE
echo "localTagList ["${localTags}"]"
tagList=${localTags//,/ }
for tag in $tagList
do
  # determine which repReqs to check
  if [[ $tag == "RM" ]] || [[ $tag == "DM" ]]; then
    checkPreReqXvfbFlag=TRUE
  fi
done

if [ "TRUE" == $checkPreReqXvfbFlag ]; then 
  # Xvfb is a system requirement
  echo "---Require Xvfb on port :3---"
  if ps -ef | grep -v "grep" | grep "Xvfb.*:3 "
  then
    echo "Xvfb [:3] already running"
  else
    echo "starting Xvfb [:3]"
    Xvfb :3 -screen 0 800x600x24 >&- 2>&- & 
    sleep 10
    
    if ps -ef | grep -v "grep" | grep "Xvfb.*:3 "
    then
      echo "Xvfb [:3] running"
    else
      echo "ERROR: Xvfb [:3] not running!"
      exit 1
    fi
  fi
  
  echo "---Setting DISPLAY---"
  DISPLAY=:3.0
  export DISPLAY
  
  
  # setting up automatically start up script for Xvfb
  checkOS
  if [ "TRUE" == $bRH6flag ]; then
    echo "*** Setup ${WAS_COMMON_DIR}/ibm-xvfb init.d"
    cp ${WAS_COMMON_DIR}/ibm-xvfb /etc/init.d
    chmod o+x /etc/init.d/ibm-xvfb
    chkconfig --add ibm-xvfb
    echo "checking start up scripts"
    ls -l /etc/init.d/ibm-xvfb
    chkconfig --list ibm-xvfb
  else
    echo "*** Setup ${WAS_COMMON_DIR}/ibm-xvfb systemd"
    if [ ! -d /usr/lib/systemd/scripts/ ];then
      mkdir /usr/lib/systemd/scripts/
    fi
    cp ${WAS_COMMON_DIR}/ibm-xvfb /usr/lib/systemd/scripts/
    chmod o+x /usr/lib/systemd/scripts/ibm-xvfb
    cp ${WAS_COMMON_DIR}/ibm-xvfb.service /usr/lib/systemd/system/
    systemctl enable ibm-xvfb.service
    echo "checking start up scripts"
    ls -l /usr/lib/systemd/scripts/ibm-xvfb
    ls -la /etc/systemd/system/multi-user.target.wants | grep ibm-xvfb.service
  fi
  
  ## ----- end of code that should eventually be moved to CoreAutomationPrep.sh -----
  ## --------------------------------------------------------------------------------
  
  ## -------------------------------------------------------------
  ## not sure this is really needed
  ## and if the above is moved to CoreAutomationPrep.sh how the DISPLAY value [3] will be passed to here?
  echo "---Adding DISPLAY to startNode.sh---"
  sed -i "s%Bootstrap values ...%ADDED FOR CONVERTER CLUSTERING SUPPORT\nDISPLAY=:3.0\nexport DISPLAY\n\n# Bootstrap values ...%" /opt/IBM/WebSphere/AppServer/bin/start*.sh
  
  ## -------------------------------------------------------------
fi


echo "----- Modify the JVM properties based on RAM size"
${WAS_COMMON_DIR}/SetCorrectHeapSize.sh
if [ $? -ne 0 ]; then
	echo "Heapsize not set in was.properties. Exiting"
	exit 1
fi

if [ "$DB_VENDOR" == "oracle" ];then
	echo "--- Modify ORACLE Driver name if DB_VENDOR=oracle"
	sed -i s%_JAR_FILE_%$dbDriverFile% ${WAS_COMMON_DIR}/was.properties
fi
echo "----- SetJvmProps"
su - $OSUserName -c "$WAS_PROFILE_ROOT/bin/wsadmin.sh -lang jython -f ${WAS_COMMON_DIR}/SetJvmProps.jy $WAS_COMMON_DIR" 
if [ $? -ne 0 ]
	then
	echo "WAS JVM settings NOT updated"
	exit 1
fi

echo "------ SetWebContainerThreadPool"
su - $OSUserName -c "$WAS_PROFILE_ROOT/bin/wsadmin.sh -lang jython -f ${WAS_COMMON_DIR}/SetThreadPoolProps.jy $WAS_COMMON_DIR" 
if [ $? -ne 0 ]
	then
	echo "WAS Webcontainer Threadpool settings NOT updated"
	exit 1
fi

echo "----- Update WAS Session Management settings"
su - $OSUserName -c "$WAS_PROFILE_ROOT/bin/wsadmin.sh -lang jython -f ${WAS_COMMON_DIR}/SetSMProps.jy $WAS_COMMON_DIR"
if [ $? -ne 0 ]
	then
	echo "WAS Session Management settings NOT updated"
	exit 1
fi
echo " "

echo $APPS |grep jts
if [ $? == 0 ]
then
   sh ${WAS_COMMON_DIR}/EnableSSO.sh export
   if [ $? -ne 0 ]
        then
        echo "JTS Single Sign on doesn't enabled"
        exit 1
   fi
fi
echo " "

echo "----- Start WAS After WAS Config"
${WAS_COMMON_DIR}/WASScripts/RestartSingleWAS.sh

echo "----- WAS settings updated"
exit 0
