#!/bin/bash

#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2016. All Rights Reserved.
#
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************

#################################################################################
##
## Name: InstallMaintenanceFilesEngine.sh $iFixConfigFilePath $iFixFilePath

##################################

## building on the prototype using a config approach
#
# --v1 tag:file:operation:destination
# --v2 tag:operation:type:source:destination
# --v3 tag:operation:type:source:destination:string

####################################

## ASSUMPTION
## 1. config file at [$iFixConfigFilePath]
## 2. patch files in a zip at [$iFixZipFileLocation]
#
#######################################

##############
## FUNCTIONS #
##############

## usage exitWithOkMsg tempString
exitWithOkMsg()
{
  tempString=$1
  echo "-- message: ${tempString}"
  echo "+++ SUCCESS : EOF [${scriptName}] +++"
  exit 0
}


## usage exitWithErrMsg tempString
exitWithErrMsg()
{
  tempString=$1
  echo "-- ERROR [${scriptName}] --"
  echo "-- message: ${tempString}"
  exit 1
}


## usage checkFileExists tempFile
checkFileExists()
{
  fe_tempFile=$1
  if [ -f ${fe_tempFile} ]; then
    echo "file [${fe_tempFile}] exists"
  else
    errMsg="file [${fe_tempFile}] does not exist"
    exitWithErrMsg "${errMsg}"
  fi
}


# tail needs altered if the inputFileName contains a partial path
updateDestinationFolderTail()
{
  tempTail=$1
  tempInputFilename=$2
  
  ## allow for potential of the [$tempInputFilename] containing a partial path
  partFile=`echo $tempInputFilename | rev | cut -d"/" -f 1 | rev`
  partFolder=`echo $tempInputFilename  | rev | cut -d"/" -f 1 --complement | rev`
  
  if [ "$partFile" == "$partFolder" ]; then
    echo "[$partFile] EQUALS [$partFolder], so [$tempTail] stays as is"
    tempDestinationFolderTail=$tempTail
  else
    echo "[$partFile] DOES NOT EQUAL [$partFolder], so [$tempTail] is updated to [$DestinationFolderTail"
    tempDestinationFolderTail=$tempTail/$partFolder
  fi
}


getFullFilename()
{
  tempInputFolder=$1
  tempInputFilename=$2
  
  ## allow for potential of the [$tempInputFilename] containing a partial path
  partFile=`echo $tempInputFilename | rev | cut -d"/" -f 1 | rev`
  partFolder=`echo $tempInputFilename  | rev | cut -d"/" -f 1 --complement | rev`
  
  if [ "$partFile" == "$partFolder" ]; then
    echo "[$partFile] EQUALS [$partFolder], so [$tempInputFilename] is just a file"
    tempFolder=$tempInputFolder
    partialFilename=$partFile
  else
    echo "[$partFile] DOES NOT EQUAL [$partFolder], so [$tempInputFilename] is folder/file"
    tempFolder=$tempInputFolder/$partFolder
    partialFilename=$partFile
  fi
  echo "so in tempFolder [$tempFolder] find a single match for partial filename [$partialFilename]"
  
  # can only be a single match
  tempFileCount=`ls ${tempFolder} | grep ${partialFilename} | wc -w`
  if [ $tempFileCount -eq 1 ]; then
    matchedFilename=`ls ${tempFolder} | grep ${partialFilename}`
    if [ $? -ne 0 ]; then
      errMsg="failed to find a match for [${partialFilename}] in folder [${tempFolder}]"
      exitWithErrMsg "${errMsg}"
    else
      echo "successfully matched [${matchedFilename}] in folder [${tempFolder}]"
      matchedPathFilename=${tempFolder}"/"${matchedFilename}
      checkFileExists $matchedPathFilename
    fi
  else
    echo "contents of [${tempFolder}]"
    ls -la ${tempFolder}
    errMsg="expected just a single file match for [${partialFilename}], found [$tempFileCount]"
    exitWithErrMsg "$errMsg"
  fi
}


getFullFoldername()
{
  tempInputFolder=$1
  tempInputFolderToCheck=$2
  
  ## allow for potential of the [$tempInputFolderToCheck] containing a partial path
  partFolderEnd=`echo $tempInputFolderToCheck | rev | cut -d"/" -f 1 | rev`
  partFolderFront=`echo $tempInputFolderToCheck  | rev | cut -d"/" -f 1 --complement | rev`
  
  if [ "$partFolderEnd" == "$partFolderFront" ]; then
    echo "[$partFolderEnd] EQUALS [$partFolderFront], so [$tempInputFolderToCheck] is just a folder"
    tempFolder=$tempInputFolder
    partialFoldername=$partFolderEnd
  else
    echo "[$partFolderEnd] DOES NOT EQUAL [$partFolderFront], so [$tempInputFolderToCheck] is folder/folder"
    tempFolder=$tempInputFolder/$partFolderFront
    partialFoldername=$partFolderEnd
  fi
  echo "so in tempFolder [$tempFolder] find a single match for partial foldername [$partialFoldername]"

  # can only be a single match
  tempFolderCount=`ls ${tempFolder} | grep ${partialFoldername} | wc -w`
  if [ $tempFolderCount -eq 1 ]; then
    matchedFoldername=`ls ${tempFolder} | grep ${partialFoldername}`
    if [ $? -ne 0 ]; then
      errMsg="failed to find a match for [${partialFoldername}] in folder [${tempFolder}]"
      exitWithErrMsg "${errMsg}"
    else
      echo "successfully matched [${matchedFoldername}] in folder [${tempFolder}]"
      matchedPathFoldername=${tempFolder}"/"${matchedFoldername}
      checkFolderExists $matchedPathFoldername
    fi
  else
    echo "contents of [${tempFolder}]"
    ls -la ${tempFolder}
    errMsg="expected just a single folder match for [${partialFoldername}], found [$tempFolderCount]"
    exitWithErrMsg "$errMsg"
  fi
}


## usage checkFolderExists tempFolder
checkFolderExists()
{
  de_tempFolder=$1
  if [ -d ${de_tempFolder} ]; then
    echo "folder [${de_tempFolder}] exists"
  else
    errMsg="folder [${de_tempFolder}] does not exist"
    exitWithErrMsg "${errMsg}"
  fi
}


isFileZipFile()
{
  case "$1" in
    *.zip)
      echo "[$1] is a zip file"
      ;;
    *)
      errMsg="[$1] is not a zip file"
      exitWithErrMsg "$errMsg"
      ;;
  esac
}


isFileTarFile()
{
  case "$1" in
    *.tar)
      echo "[$1] is a tar file"
      ;;
    *)
      errMsg="[$1] is not a tar file"
      exitWithErrMsg "$errMsg"
      ;;
  esac
}


# TRUE | FALSE is there just one file in the folder
justOneFile()
{
  # folder=$1
  tempFileCount=`ls $1 | wc -w`
  if [ $tempFileCount -eq 1 ]; then
    echo "folder [$1] contains a single file"
  else
    echo "contents of [$1]"
    ls -la $1
    errMsg="expected just a single file, found [$tempFileCount]"
    exitWithErrMsg "$errMsg"
  fi
}


## make a new folder including any missing folders in the path
makeFolderFullPath()
{
  newFolder=$1
  echo "make new folder (inc path) [${newFolder}]"
  mkdir -p ${newFolder}
  checkFolderExists ${newFolder}
}


trimWhitespace()
{
    local var="$*"
    var="${var#"${var%%[![:space:]]*}"}"   # remove leading whitespace characters
    var="${var%"${var##*[![:space:]]}"}"   # remove trailing whitespace characters
    echo -n "$var"
}

# string being trimmed could be in the format
# - {whitespace "[" whitespace value whitespace}
# - only want the {value part}
trim()
{
    local var="$*"
    var="${var#"${var%%[![:space:]]*}"}"   # remove leading whitespace characters
    var="${var#"["}"   # remove any leading "[" character
    var="${var#"${var%%[![:space:]]*}"}"   # remove leading whitespace characters
    var="${var%"${var##*[![:space:]]}"}"   # remove trailing whitespace characters
    echo -n "$var"
}


# copy all the contents from the current folder to the destination folder, which must exist
copyFolderContents()
{
  tempDestinationFolder=$1
  
  echo "copy current folder contents to ["${tempDestinationFolder}"]"
  echo "current folder contents"
  ls -la
  
  # warn if folder in empty, but do not error
  tempFileCount=`ls | wc -w`
  if [ $tempFileCount -eq 0 ]; then
    echo "WARNING: there are no folders/files in the current folder to copy"
    echo "         not attempting to copy anything to [${tempDestinationFolder}]"
  else
    cp -prf * ${tempDestinationFolder}
    if [ $? == 0 ]; then
      echo "copy complete"
      echo "destination folder contents"
      ls -la ${tempDestinationFolder}
    else
      errMsg="copy failed"
      exitWithErrMsg "${errMsg}"
    fi
  fi
}


## usage copyFile (force) from current folder to folder
copyFile()
{
  tempFile=$1
  tempFolder=$2
  echo "copy ["${tempFile}"] to ["${tempFolder}/${tempFile}"]"
  cp -pf ${tempFile} ${tempFolder}/${tempFile}
  if [ $? == 0 ]; then
    echo "copy complete" 
    if [ -f ${tempFolder}/${tempFile} ]; then
      echo "copy successful, file exists"
    else
      errMsg="file does not exist"
      exitWithErrMsg "${errMsg}"
    fi
  else
    errMsg="copy failed"
    exitWithErrMsg "${errMsg}"
  fi
}


## usage copyFile (force) from current folder to folder
copyFileNewName()
{
  tempFile=$1
  tempNewFile=$2
  tempFolder=$3
  echo "copy ["${tempFile}"] to ["${tempFolder}/${tempNewFile}"]"
  cp -pf ${tempFile} ${tempFolder}/${tempNewFile}
  if [ $? == 0 ]; then
    echo "copy complete" 
    if [ -f ${tempFolder}/${tempNewFile} ]; then
      echo "copy successful, file exists"
    else
      errMsg="file does not exist"
      exitWithErrMsg "${errMsg}"
    fi
  else
    errMsg="copy failed"
    exitWithErrMsg "${errMsg}"
  fi
}


## usage deleteFile (force) from folder
deleteFile()
{
  tempFolder=$1
  tempFile=$2
  fileToDelete=${tempFolder}"/"${tempFile}

  if [ -f ${fileToDelete} ]; then
    echo "delete file ["${fileToDelete}"]"
    rm -f ${fileToDelete}
    if [ -f ${fileToDelete} ]; then
      errMsg="failed to delete file"
      exitWithErrMsg "${errMsg}"
    else
      echo "succesfully deleted [${fileToDelete}]"
    fi
  else
    echo "file [${fileToDelete}] does not exist, so cannot be deleted"
  fi
}


## usage recursively deleteFolder (force)
deleteFolder()
{
  tempFolder=$1
  
  if [ -d ${tempFolder} ]; then
    echo "delete folder ["${tempFolder}"]"
    rm -Rf ${tempFolder}
    if [ -d ${tempFolder} ]; then
      errMsg="failed to delete folder"
      exitWithErrMsg "${errMsg}"
    else
      echo "succesfully deleted [${tempFolder}]"
    fi
  else
    echo "folder [${tempFolder}] does not exist, so cannot be deleted"
  fi
}


# get file attributes
getFileAttributes()
{
  tempCurrentFolder=$1
  tempFile=$2
  currentFile=${tempCurrentFolder}"/"${tempFile}
  
  # if file exists get existing file attributes
  attributeFlag=FALSE
  if [ -f ${currentFile} ]; then
    echo "file [${currentFile}] already exists, getting attributes for later"
    attributeUser=`ls -l ${currentFile} | cut -d' ' -f 3`
    attributeGroup=`ls -l ${currentFile} | cut -d' ' -f 4`
    attributePermissions=`stat -c "%a" ${currentFile}`
    attributeFlag=TRUE
    echo "captured attributes [owner]:[${attributeUser}] [group]:[${attributeGroup}] [permissions]:[${attributePermissions}]"

  else
    echo "file [${currentFile}] does not exist, will use the permissions of current user"
    currentAttributes=`id`
    echo "current user attributes [${currentAttributes}]"
    attributeFlag=FALSE
  fi
}


# get folder attributes
getFolderAttributes()
{
  tempCurrentFolder=$1
  
  # if folder exists get existing folder attributes
  attributeFlag=FALSE
  if [ -d ${tempCurrentFolder} ]; then
    echo "folder [${tempCurrentFolder}] already exists, getting attributes for later"
    attributeUser=`ls -ld ${tempCurrentFolder} | cut -d' ' -f 3`
    attributeGroup=`ls -ld ${tempCurrentFolder} | cut -d' ' -f 4`
    attributePermissions=`stat -c "%a" ${tempCurrentFolder}`
    attributeFlag=TRUE
    echo "captured attributes [owner]:[${attributeUser}] [group]:[${attributeGroup}] [permissions]:[${attributePermissions}]"

  else
    echo "folder [${tempCurrentFolder}] does not exist, will use the permissions of current user"
    currentAttributes=`id`
    echo "current user attributes [${currentAttributes}]"
    attributeFlag=FALSE
  fi
}


# set file attributes
# expects the following already set 
# $attributeFlag [TRUE attributes exist | FALSE none]
# $attributeUser $attributeGroup $attributePermissions
setFileAttributes()
{
  tempCurrentFolder=$1
  tempFile=$2
  currentFile=${tempCurrentFolder}"/"${tempFile}
  
  checkFileExists ${currentFile}
  echo "current attributes"
  ls -la ${currentFile}
    
  # if file previously existed replace its permissions with the original ones
  if [ "TRUE" == ${attributeFlag} ]; then
    echo "replace file permissions with the original ones [owner][group][permissions]"
    echo "[${attributeUser}][${attributeGroup}][${attributePermissions}]"
    chmod ${attributePermissions} ${currentFile}
    chgrp ${attributeGroup} ${currentFile}
    chown ${attributeUser} ${currentFile}
    echo "updated attributes"
    ls -la ${currentFile}
  fi
}


# recursively set file attributes starting wit the target folder name 
# expects the following already set 
# $attributeFlag [TRUE attributes exist | FALSE none]
# $attributeUser $attributeGroup $attributePermissions
setFolderAttributes()
{
  tempCurrentFolder=$1
  
  checkFolderExists ${tempCurrentFolder}
  echo "current attributes of top level folder [${tempCurrentFolder}]"
  ls -ld ${tempCurrentFolder}
    
  # if file previously existed replace its permissions with the original ones
  if [ "TRUE" == ${attributeFlag} ]; then
    echo "recursively replace all folder file permissions with the original ones [owner][group][permissions]"
    echo "[${attributeUser}][${attributeGroup}][${attributePermissions}]"
    chmod --recursive ${attributePermissions} ${tempCurrentFolder}
    chgrp --recursive ${attributeGroup} ${tempCurrentFolder}
    chown --recursive ${attributeUser} ${tempCurrentFolder}
    echo "updated attributes"
    ls -la ${tempCurrentFolder}
  fi
}


# call correct deleteAction function based on "type"
doDeleteAction()
{
  tempType=$1
  tempFile=$2
  tempDestinationFolder=$3

  if [ "file" == $tempType ]; then
    # delete the file in the folder
    doDeleteFileAction $tempFile $tempDestinationFolder
  elif [ "folder" == $tempType ]; then
    # foldername is contained in $tempFile
    sourceFolder=$tempDestinationFolder"/"$tempFile
    # delete the source folder
    doDeleteFolderAction $sourceFolder
  else
    errMsg="unsupported type [${tempType}]"
    exitWithErrMsg "$errMsg"
  fi

}


doDeleteFileAction()
{
  tempFile=$1
  tempDestinationFolder=$2
 
  tempDeleteFile=${tempDestinationFolder}"/"${tempFile}
 
  echo "doDeleteFileAction on [${tempDeleteFile}]"
  checkFileExists ${tempDeleteFile}
  # delete existing file  
  echo "delete existing [$tempDeleteFile] file"
  deleteFile ${tempDestinationFolder} ${tempFile}
}


doDeleteFolderAction()
{
  tempDeleteFolder=$1
  
  echo "doDeleteFolderAction on [${tempDeleteFolder}]"
  checkFolderExists ${tempDeleteFolder}
  # delete existing destination folder contents
  echo "delete existing folder [$tempDeleteFolder]"
  deleteFolder ${tempDeleteFolder}
}


# call correct copyAction function based on "type"
doCopyAction()
{
  tempType=$1
  tempCurrentFolder=$2
  tempFile=$3
  tempDestinationFolder=$4

  if [ "file" == $tempType ]; then
    # copy the file to the folder, without saving any existing version
    doCopyFileAction $tempCurrentFolder $tempFile $tempDestinationFolder
  elif [ "folder" == $tempType ]; then
    # copy the source folder contents to the destination folder, without saving any existing content
    # foldername is contained in $tempFile
    sourceFolder=$tempCurrentFolder"/"$tempFile
    checkFolderExists $sourceFolder
    cd $sourceFolder
    echo "doCopyFolderAction $sourceFolder $tempDestinationFolder"
    doCopyFolderAction $sourceFolder $tempDestinationFolder
    
  else
    errMsg="unsupported type [${tempType}]"
    exitWithErrMsg "$errMsg"
  fi

}


# copy $tempFile into $tempDestinationFolder
# create the folder if it does not exist
# do not keep a copy of the current file
doCopyFileAction()
{
  ca_tempCurrentFolder=$1
  ca_tempFile=$2
  ca_tempDestinationFolder=$3
 
  ca_currentFile=${ca_tempCurrentFolder}"/"${ca_tempFile}
  ca_newFile=${ca_tempDestinationFolder}"/"${ca_tempFile}
 
  echo "doCopyFileAction on [${ca_currentFile}] with result in [${ca_tempDestinationFolder}]"
  checkFileExists ${ca_currentFile}
  ## ensure target folder exists
  makeFolderFullPath ${ca_tempDestinationFolder}
  # if file exists get existing file attributes
  getFileAttributes ${ca_tempDestinationFolder} ${ca_tempFile}
  
  # delete existing file  
  echo "delete existing [$ca_newFile}] file"
  deleteFile ${ca_tempDestinationFolder} ${ca_tempFile}
  # copy the new file from the current folder
  cd ${ca_tempCurrentFolder}
  copyFile ${ca_tempFile} ${ca_tempDestinationFolder}
  checkFileExists ${ca_newFile}
  ls -la ${ca_newFile}
  # if file previously existed replace its attributes with the original ones
  setFileAttributes ${ca_tempDestinationFolder} ${ca_tempFile}

}


# copy contents of the current folder into $tempDestinationFolder
# create the folder if it does not exist
# do not keep a copy of the current files
doCopyFolderAction()
{
 ca_tempCurrentFolder=$1
 ca_tempDestinationFolder=$2
 
 echo "doCopyFolderAction on [${ca_tempCurrentFolder}] with result in [${ca_tempDestinationFolder}]"
  
  # if folder exists get existing folder permissions
  getFolderAttributes ${ca_tempDestinationFolder}

  # delete existing destination folder contents
  echo "delete existing folder [$ca_tempDestinationFolder]"
  deleteFolder ${ca_tempDestinationFolder}
  ## ensure target folder exists
  makeFolderFullPath  ${ca_tempDestinationFolder}

  # copy the current folder contents
  cd ${ca_tempCurrentFolder}
  copyFolderContents ${ca_tempDestinationFolder}
  checkFolderExists ${ca_tempDestinationFolder}
  ls -la ${ca_tempDestinationFolder}
  
  # if folder previously existed recursively replace its permissions with the original ones
  setFolderAttributes ${ca_tempDestinationFolder}

}


# call correct replaceAction function based on "type"
doReplaceAction()
{
  tempType=$1
  tempCurrentFolder=$2
  tempFile=$3
  tempDestinationFolder=$4
  tempSaveInFolder=$5

  if [ "file" == $tempType ]; then
    # copy the file to the folder, but first save any existing version
    doReplaceFileAction $tempCurrentFolder $tempFile $tempDestinationFolder $tempSaveInFolder
  elif [ "folder" == $tempType ]; then
    # copy the source folder content to the destination folder, but first save any existing content
    # foldername is contained in $tempFile
    sourceFolder=$tempCurrentFolder"/"$tempFile
    checkFolderExists $sourceFolder
    cd $sourceFolder
    echo "doReplaceFolderAction $sourceFolder $tempDestinationFolder $tempSaveInFolder"
    doReplaceFolderAction $sourceFolder $tempDestinationFolder $tempSaveInFolder
    
  else
    errMsg="unsupported type [${tempType}]"
    exitWithErrMsg "$errMsg"
  fi

}


# copy $tempFile into $tempDestinationFolder
# create the folder if it does not exist
# make a copy of the current file first
doReplaceFileAction()
{
  ra_tempCurrentFolder=$1
  ra_tempFile=$2
  ra_tempDestinationFolder=$3
  ra_tempSaveInFolder=$4
  
  ra_currentFile=${ra_tempCurrentFolder}"/"${ra_tempFile}
  ra_newFile=${ra_tempDestinationFolder}"/"${ra_tempFile}
  ra_savedFile=${ra_tempSaveInFolder}"/"${ra_tempFile}
  
  echo "doReplaceFileAction on [${ra_tempFile}] with result in [${ra_tempDestinationFolder}]"
  #' ensure the file to be copied exists
  checkFileExists ${ra_currentFile}
  # save the existing file
  echo "save the existing file [${ra_newFile}] file in [${ra_tempSaveInFolder}]"
  doCopyFileAction ${ra_tempDestinationFolder} ${ra_tempFile} ${ra_tempSaveInFolder}
  # (replace) copy in the new file
  echo "now copy the new file [${ra_currentFile}] in place of the existing one in [${ra_tempDestinationFolder}]"
  doCopyFileAction ${ra_tempCurrentFolder} ${ra_tempFile} ${ra_tempDestinationFolder}
  
}


# copy $tempCurrentFolder contents into $tempDestinationFolder
# create the folder if it does not exist
# make a copy of the current folder contents first
doReplaceFolderAction()
{
  ra_tempCurrentFolder=$1
  ra_tempDestinationFolder=$2
  ra_tempSaveInFolder=$3
  
  echo "doReplaceFolderAction on [${ra_tempCurrentFolder}] with result in [${ra_tempDestinationFolder}]"
  
  # save the existing folder contents
  echo "make a copy of existing folder [${ra_tempDestinationFolder}] in [${ra_tempSaveInFolder}]"
  cd ${ra_tempDestinationFolder}
  doCopyFolderAction ${ra_tempDestinationFolder} ${ra_tempSaveInFolder}
  # back to current folder
  cd ${ra_tempCurrentFolder}
  echo "now copy the new folder contents in place of the existing contents"
  doCopyFolderAction ${ra_tempCurrentFolder} ${ra_tempDestinationFolder}
  
}


# call correct backupAction function based on "type"
doBackupAction()
{
  tempType=$1
  tempFile=$2
  tempSourceFolder=$3
  tempSaveInFolder=$4

  if [ "file" == $tempType ]; then
    # save a copy of the file
    doBackupFileAction $tempFile $tempSourceFolder $tempSaveInFolder
  elif [ "folder" == $tempType ]; then
    # save a copy of the source folder
    # foldername is contained in $tempFile
    sourceFolder=$tempSourceFolder"/"$tempFile
    destinationFolder=$tempSaveInFolder"/"$tempFile
    checkFolderExists $sourceFolder
    cd $sourceFolder
    doBackupFolderAction $sourceFolder $destinationFolder
    
  else
    errMsg="unsupported type [${tempType}]"
    exitWithErrMsg "$errMsg"
  fi

}


# copy $ba_tempFile into $ba_tempSaveInFolder
# create the folder if it does not exist
# make a copy of the current file first
doBackupFileAction()
{
  ba_tempFile=$1
  ba_tempSourceFolder=$2
  ba_tempSaveInFolder=$3
  
  ba_currentFile=${ba_tempSourceFolder}"/"${ba_tempFile}
  ba_savedFile=${ba_tempSaveInFolder}"/"${ba_tempFile}
  
  echo "doBackupFileAction on [${ba_currentFile}] with result in [${ba_tempSaveInFolder}]"
  #' ensure the file to be copied exists
  checkFileExists ${ba_currentFile}
  # save the existing file
  echo "save the existing file [${ba_currentFile}] file in [${ba_tempSaveInFolder}]"
  doCopyFileAction ${ba_tempSourceFolder} ${ba_tempFile} ${ba_tempSaveInFolder}
}


# copy $ba_tempSourceFolder contents into $ba_tempSaveInFolder
# create the folder if it does not exist
# make a copy of the current folder contents first
doBackupFolderAction()
{
  ba_tempSourceFolder=$1
  ba_tempSaveInFolder=$2
  
  echo "doBackupFolderAction on [${ba_tempSourceFolder}] with result in [${ba_tempSaveInFolder}]"
  
  # save the existing folder contents
  echo "make a copy of existing folder [${ba_tempSourceFolder}] in [${ba_tempSaveInFolder}]"
  cd ${ba_tempSourceFolder}
  doCopyFolderAction ${ba_tempSourceFolder} ${ba_tempSaveInFolder}
}











# call correct copyAction function based on "type"
# note as the tar/zip file has previously been extracted to teh temp folder one function can satisfy both
#doUnzipCopyAction()
doExtractCopyAction()
# call correct replaceCopy function based on "type"
{
  tempType=$1
  tempCurrentFolder=$2
  tempDestinationFolder=$3

  cd $tempCurrentFolder
  if [ "file" == $tempType ]; then
    # there should only be one file in the folder, but have to get its name
    justOneFile $tempCurrentFolder
    tempFileName=`ls`
    tempFullFilename=$tempCurrentFolder/$tempFileName
    echo "full path to new file [${tempFileName}] [${tempFullFilename}]"
    checkFileExists ${tempFullFilename}
    
    doCopyFileAction $tempCurrentFolder $tempFileName $tempDestinationFolder
    
  elif [ "files" == $tempType ]; then
    # have to loop across each file in the folder
    tempFileFolder=$tempCurrentFolder
    echo "==== processing the contents of folder [$tempFileFolder] ===="
    tempFileList=`ls`
    read -r -a tempFileArray <<< ${tempFileList}
    for tempFileName in "${tempFileArray[@]}"
    do
      echo "tempFileName = [$tempFileName]"
      if [ -f $tempFileName ]; then
        tempFullFilename=$tempFileFolder/$tempFileName
        echo "full path to new file [${tempFileName}] [${tempFullFilename}]"
        checkFileExists ${tempFullFilename}
        
        doCopyFileAction $tempFileFolder $tempFileName $tempDestinationFolder
      else
        echo "[$tempFileName] not a file"
      fi
    done
  
  elif [ "folder" == $tempType ]; then
    doCopyFolderAction $tempCurrentFolder $tempDestinationFolder
  else
    errMsg="unsupported type [${tempType}]"
    exitWithErrMsg "$errMsg"
  fi
}



# call correct replaceAction function based on "type"
# note as the tar/zip file has previously been extracted to teh temp folder one function can satisfy both
#doUnzipReplaceAction()
doExtractReplaceAction()
{
  tempType=$1
  tempCurrentFolder=$2
  tempDestinationFolder=$3
  tempSaveInFolder=$4

  cd $tempCurrentFolder
  if [ "file" == $tempType ]; then
    # there should only be one file in the folder, but have to get its name
    justOneFile $tempCurrentFolder
    tempFileName=`ls`
    tempFullFilename=$tempCurrentFolder/$tempFileName
    echo "full path to new file [${tempFileName}] [${tempFullFilename}]"
    checkFileExists ${tempFullFilename}
    
    doReplaceFileAction $tempCurrentFolder $tempFileName $tempDestinationFolder $tempSaveInFolder
    
  elif [ "files" == $tempType ]; then
    # have to loop across each file in the folder
    tempFileFolder=$tempCurrentFolder
    echo "==== processing the contents of folder [$tempFileFolder] ===="
    tempFileList=`ls`
    read -r -a tempFileArray <<< ${tempFileList}
    for tempFileName in "${tempFileArray[@]}"
    do
      echo "tempFileName = [$tempFileName]"
      if [ -f $tempFileName ]; then
        tempFullFilename=$tempFileFolder/$tempFileName
        echo "full path to new file [${tempFileName}] [${tempFullFilename}]"
        checkFileExists ${tempFullFilename}
        
        doReplaceFileAction $tempFileFolder $tempFileName $tempDestinationFolder $tempSaveInFolder
      else
        echo "[$tempFileName] not a file"
      fi
    done
 
  elif [ "folder" == $tempType ]; then
    # replace the entire contents of the $tempDestinationFolder with the entire contents of the $tempCurrentFolder
    doReplaceFolderAction $tempCurrentFolder $tempDestinationFolder $tempSaveInFolder
    
  else
    errMsg="unsupported type [${tempType}]"
    exitWithErrMsg "$errMsg"
  fi
}


## usage unzip contents to target folder
## using the [-o] option to force overwriting of any existing files
## (also makes it the same as the "tar x" default)
unzipZip()
{
  zipFile=$1
  targetDir=$2
  
  isFileZipFile ${zipFile}
  
  unzip -o ${zipFile} -d ${targetDir}
  if [ $? == 0 ]; then
    echo "unzip successful" 
  else
    errMsg="failed to unzip  ["${zipFile}"]"
    exitWithErrMsg "$errMsg"
  fi
}

## usage "untar" contents to target folder
## using defaults which overwrites any existing files
untarTar()
{
  tarFile=$1
  targetDir=$2
  
  isFileTarFile ${tarFile}
  
  tar xpf ${tarFile} -C ${targetDir}
  if [ $? == 0 ]; then
    echo "tar extract successful" 
  else
    errMsg="failed to tar extract ["${tarFile}"]"
    exitWithErrMsg "$errMsg"
  fi
}


## modify the teamserver.properties file
doModifyTspKey()
{
  tempKeyValuePair=$1
  tempTspFile=$2
  tempDestinationFolder=$3
  tempSaveInFolder=$4

  # save the existing file
  echo "save the existing file [${tempFile}] file in [${tempSaveInFolder}]"
  doCopyFileAction ${tempDestinationFolder} ${tempTspFile} ${tempSaveInFolder}
  
  # TSP file history is also kept
  if [ "teamserver.properties" == ${tempTspFile} ]; then
    tempTspNewFile="teamserver-iFix-"$timestamp".properties"
    echo "copy current [teamserver.properties] to ["$tempDestinationFolder"/"$tempTspNewFile"]"
    copyFileNewName $tempTspFile $tempTspNewFile $tempDestinationFolder
  fi
  
  # extract the key and value
  tempTspKey=`echo $tempKeyValuePair | cut -d'=' -f 1`
  tempTspValue=`echo $tempKeyValuePair | cut -d'=' -f 1 --complement`
  
  # get existing file attributes
  getFileAttributes ${tempDestinationFolder} ${tempTspFile}
  # update the stored pair
  updateKey $tempDestinationFolder"/"$tempTspFile $tempTspKey $tempTspValue
  # replace its attributes with the original ones
  setFileAttributes ${tempDestinationFolder} ${tempTspFile}

}


## modify the teamserver.properties file
doDeleteTspKey()
{
  tempTspKey=$1
  tempTspFile=$2
  tempDestinationFolder=$3
  tempSaveInFolder=$4

  # save the existing file
  echo "save the existing file [${tempFile}] file in [${tempSaveInFolder}]"
  doCopyFileAction ${tempDestinationFolder} ${tempTspFile} ${tempSaveInFolder}
  
  # TSP file history is also kept
  if [ "teamserver.properties" == ${tempTspFile} ]; then
    tempTspNewFile="teamserver-iFix-"$timestamp".properties"
    echo "copy current [teamserver.properties] to ["$tempDestinationFolder"/"$tempTspNewFile"]"
    copyFileNewName $tempTspFile $tempTspNewFile $tempDestinationFolder
  fi
  
  # get existing file attributes
  getFileAttributes ${tempDestinationFolder} ${tempTspFile}
  # delete the stored pair
  deleteKey $tempDestinationFolder"/"$tempTspFile $tempTspKey
  # replace its attributes with the original ones
  setFileAttributes ${tempDestinationFolder} ${tempTspFile}

}


# update property file with the new key:value pair
updateKey()
{
  origFile=$1
  tempKey=$2
  newValue=$3

  tempFile=${origFile}.tmp
  ls -la $origFile

  matchKey=${tempKey}"="
  # does key already exist?
  cat ${origFile} | grep "^${matchKey}" > /dev/null
  if [ $? -gt 0 ]; then
    # key does not exist, so add it to the file
    echo "add new key ["${tempKey}"] ["${newValue}"]"
    echo ${tempKey}"="${newValue}>> ${origFile}
  else
    # key does exist, so update it
    echo "update existing key with ["${tempKey}"] ["${newValue}"]"
    # lines starting with # should not be updated
    sed "s/^ *${tempKey}=.*/${tempKey}=$newValue/g" ${origFile} > ${tempFile}
    if [ $? == 0 ]; then
      echo "sed replace successful"
      mv ${tempFile} ${origFile}
    else
      errMsg="failed sed replace on ["${origFile}"]"
      exitWithErrMsg "$errMsg"
    fi
    
  fi

  # prove that the file has changed
  ls -la $origFile
}

# update property file by deleting key:value pair
deleteKey()
{
  origFile=$1
  tempKey=$2

  tempFile=${origFile}.tmp
  ls -la $origFile

  matchKey=${tempKey}"="
  # does key already exist?
  cat ${origFile} | grep "^${matchKey}" > /dev/null
  if [ $? -gt 0 ]; then
    echo "key ${tempKey} does not exist, so no need to delete it"
  else
    # key does exist, so delete it
    echo "deleting existing key(s) with ["${tempKey}"]"
    # lines starting with # should not be deleted
    sed "/^ *${tempKey}=.*/d" ${origFile} > ${tempFile}
    if [ $? == 0 ]; then
      echo "sed delete successful"
      mv ${tempFile} ${origFile}
    else
      errMsg="failed sed delete on ["${origFile}"]"
      exitWithErrMsg "$errMsg"
    fi
  fi

  # prove that the file has changed
  ls -la $origFile
}





############
### MAIN ###
############

scriptName=$0
workingDir=`pwd -P`
timestamp=`date +%s`

# get env properties
envProp=/etc/env.properties
. $envProp

# get media properties
mediaProp=$AUTO_PREP/MediaParameters/media.properties
. $mediaProp

# target folders for iFix files
CLM_HOME=/opt/IBM/JazzTeamServer

#localIFixHeadFolder=${QD_InternalMedia}"/"${iFixMediaRoot}
# - will contain the extracted content of the iFix zip
#localIFixZipFolderFiles=${localIFixHeadFolder}"/Files"
#makeFolderFullPath ${localIFixZipFolderFiles}

savedFilesFolderHead=$CLM_HOME/server/replacedByIFIX_$timestamp


## ======================================================================
## check input parameters
#  ======================
filename=$1
zipFile=$2

echo "checking inputs parameters"
if [ $# != 2 ]; then
  echo "ERROR: script "${scriptName}" requires 2 parameters: [path to config filename] [path to iFix zip file]"
  echo "not $0 $@"
  errMsg="command line syntax error"
  exitWithErrMsg "$errMsg"
fi

# check files exist
checkFileExists ${zipFile}
checkFileExists ${filename}


## ======================================================================
## unzip the iFix patch zip file
#  =============================
echo "unzip the iFix patch file to [$localIFixZipFolderFiles]"
unzipZip ${zipFile} $localIFixZipFolderFiles
ls -la $localIFixZipFolderFiles



## ======================================================================
## parse config file and install iFix
#  ==================================

# -- while config file open
# -- read one line at a time
# --CURRENT-- tag:operation:type:source:destination:string

# -- strip leading / trailing whitespace
# -- tokenise the line (split on colons [:])
# -- token 1 could be a csv
# -- if token 1 exists in the current server's list of tags continue
# --++ carry out $operation on $file putting result in $destination


numberOfTokens=6
echo "parsing file [${filename}]"


# strip leading / trailing whitespace -- while read -r inputLine
# do NOT strip leading / trailing whitespace -- while IFS='' read -r inputLine
while read -r inputLine
do
  echo "------------------------------------------------------"
  echo "input line [${inputLine}]"
  
  firstChar=`echo ${inputLine} | cut -c -1`
  echo "first char [${firstChar}]"
  
  # ignore blank lines
  wordCount=`echo ${inputLine} | wc -w`
  # echo "word count [$wordCount]"
  if [ $wordCount -eq 0 ]; then
    echo "skip blank line"
  elif [ "#" == $firstChar ]; then
    echo "skip comment line [${inputLine}]"
  else 
  
    ## split line into tokens
    ## - each token will be ".value." where . is potential whitespace
    preTokenIFS=$IFS
    export IFS=":"
  
    # there must be [$numberOfTokens] tokens, unless it is a comment starting with a [#]
    tokenCount=0
    for token in ${inputLine}
    do
      tokenCount=$((tokenCount + 1))
    done
    if [ ${tokenCount} -ne $numberOfTokens ]; then
      errMsg="wrong number of tokens, there should be $numberOfTokens not [${tokenCount}]"
      exitWithErrMsg "$errMsg"
    else
      # make an array of tokens
      tokenArray=( $inputLine )
    fi
    
    # remove whitespace
    # (use array of indexes rather than array of values)
    for index in "${!tokenArray[@]}"
    do
    #  echo "[$index][${tokenArray[index]}]"
      tokenArray[index]=`trimWhitespace ${tokenArray[index]}`
    #  echo "trimmed value [${tokenArray[index]}]"
    done
  
    # allocate tokens to variables
    # --++ tag:operation:type:source:destination:string
    operationTags=${tokenArray[0]}
    operationAction=${tokenArray[1]}
    operationDestinationType=${tokenArray[2]}
    operationFile=${tokenArray[3]}
    operationDestinationFolderTail=${tokenArray[4]}
    operationMiscString=${tokenArray[5]}
    echo "[tag] [$operationTags]"
    echo "[action] [$operationAction]"
    echo "[type] [$operationDestinationType]"
    echo "[source] [$operationFile]"
    echo "[destination] [$operationDestinationFolderTail]"
    echo "[misc string (for the future)] [$operationMiscString]"
    
    # update $operationDestinationFolder as it is relative to $CLM_HOME
    echo "front [destination] with [${CLM_HOME}]"
    operationDestinationFolder=${CLM_HOME}/$operationDestinationFolderTail
    echo "[destination] [$operationDestinationFolder]"
    
    # is the $operationTags valid for the current server
    echo "current server supports [${APPS}] tags"
    validServerFlag="FALSE"
    
    preTagsIFS=$IFS
    export IFS=","
    for tO in ${operationTags}
    do
      for tS in ${APPS}
      do
        if [ "$tO" == "$tS" ]; then
          echo "match: operationTag [$tO] equals supportedTag [$tS]"
          validServerFlag="TRUE"
        fi
      done
    done
    IFS=$preTagsIFS
    IFS=$preTokenIFS
    
    
    # check $operationDestinationType is valid
    if [ "file" == ${operationDestinationType} ] || [ "files" == ${operationDestinationType} ] || [ "folder" == ${operationDestinationType} ]; then
      echo "valid type [${operationDestinationType}]"
    else
      errMsg="unsupported type [${operationDestinationType}]"
      exitWithErrMsg "$errMsg"
    fi
    
    
    # only process the operation if a tag match happened
    if [ "FALSE" == $validServerFlag ]; then
      echo "tag [${operationTags}] not valid for current server, will not carry out operation"
    else
      echo "tag [${operationTags}] valid for current server, will now carry out operation"
      
      if [ "delete" == $operationAction ] || [ "backup" == $operationAction ] || [ "backupDelete" == $operationAction ]; then
        if [ "folder" == $operationDestinationType ]; then
          # note $operationFile contains the folder name
          getFullFoldername ${operationDestinationFolder} $operationFile
          echo "full pathname of operational file(folder) [${operationFile}] [${matchedPathFoldername}]"
          actualDestinationFolder=$(dirname ${matchedPathFoldername})
          actualOperationFile=$(basename ${matchedPathFoldername})
          echo "full path to operational file(folder) [${operationFile}] [${actualDestinationFolder}]"
          echo "full name of operational file(folder) [${operationFile}] [${actualOperationFile}]"
        elif [ "files" == $operationDestinationType ]; then
          errMsg="[$operationAction] operation : unsupported type [${operationDestinationType}]"
          exitWithErrMsg "$errMsg"
        else
          # does file exist?
          getFullFilename ${operationDestinationFolder} $operationFile
          echo "full pathname of operational file [${operationFile}] [${matchedPathFilename}]"
          actualDestinationFolder=$(dirname ${matchedPathFilename})
          actualOperationFile=$(basename ${matchedPathFilename})
          echo "full path to operational file [${operationFile}] [${actualDestinationFolder}]"
          echo "full name of operational file [${operationFile}] [${actualOperationFile}]"
        fi
        # potentially need to update the "DestinationFolderTail" used later
        updateDestinationFolderTail $operationDestinationFolderTail $operationFile
        actualDestinationFolderTail=$tempDestinationFolderTail

       elif [ "copy" == $operationAction ] || [ "replace" == $operationAction ]; then
        if [ "folder" == $operationDestinationType ]; then
          # note $operationFile contains the folder name to copy the contents of
          getFullFoldername ${localIFixZipFolderFiles} $operationFile
          echo "full pathname of operational file(folder) [${operationFile}] [${matchedPathFoldername}]"
          actualOperationFolder=$(dirname ${matchedPathFoldername})
          actualOperationFile=$(basename ${matchedPathFoldername})
          echo "full path to operational file(folder) [${operationFile}] [${actualOperationFolder}]"
          echo "full name of operational file(folder) [${operationFile}] [${actualOperationFile}]"
        elif [ "files" == $operationDestinationType ]; then
          errMsg="[$operationAction] operation : unsupported type [${operationDestinationType}]"
          exitWithErrMsg "$errMsg"
        else
          # does file exist in iFix?
          getFullFilename ${localIFixZipFolderFiles} $operationFile
          echo "full pathname of operational file [${operationFile}] [${matchedPathFilename}]"
          actualOperationFolder=$(dirname ${matchedPathFilename})
          actualOperationFile=$(basename ${matchedPathFilename})
          echo "full path to operational file [${operationFile}] [${actualOperationFolder}]"
          echo "full name of operational file [${operationFile}] [${actualOperationFile}]"
        fi
        
      elif [ "unzip" == $operationAction ] || [ "unzipSameDir" == $operationAction ] || [ "untar" == $operationAction ] || [ "untarSameDir" == $operationAction ]; then
        if [ "folder" == $operationDestinationType ] || [ "files" == $operationDestinationType ]; then
          errMsg="[$operationAction] operation : unsupported type [${operationDestinationType}]"
          exitWithErrMsg "$errMsg"
        else
          # does file exist in iFix?
          getFullFilename ${localIFixZipFolderFiles} $operationFile
          echo "full pathname of operational file [${operationFile}] [${matchedPathFilename}]"
          actualOperationFolder=$(dirname ${matchedPathFilename})
          actualOperationFile=$(basename ${matchedPathFilename})
          echo "full path to operational file [${operationFile}] [${actualOperationFolder}]"
          echo "full name of operational file [${operationFile}] [${actualOperationFile}]"
        fi
        
      elif [ "modifyTspKey" == $operationAction ] || [ "deleteTspKey" == $operationAction ]; then
        if [ "folder" == $operationDestinationType ] || [ "files" == $operationDestinationType ]; then
          errMsg="[$operationAction] operation : unsupported type [${operationDestinationType}]"
          exitWithErrMsg "$errMsg"
        else
          # does file exist on server?
          getFullFilename ${operationDestinationFolder} $operationFile
          echo "full pathname of operational file [${operationFile}] [${matchedPathFilename}]"
          actualDestinationFolder=$(dirname ${matchedPathFilename})
          actualOperationFile=$(basename ${matchedPathFilename})
          echo "full path to TSP file to modify [${operationFile}] [${actualDestinationFolder}]"
          echo "full name of TSP file to modify [${operationFile}] [${actualOperationFile}]"
          # potentially need to update the "DestinationFolderTail" used later
          updateDestinationFolderTail $operationDestinationFolderTail $operationFile
          actualDestinationFolderTail=$tempDestinationFolderTail
        fi
        
      elif [ "unzipCopy" == $operationAction ] || [ "unzipReplace" == $operationAction ] || [ "untarCopy" == $operationAction ] || [ "untarReplace" == $operationAction ]; then
        # does file exist in iFix?
        getFullFilename ${localIFixZipFolderFiles} $operationFile
        echo "full pathname of operational file [${operationFile}] [${matchedPathFilename}]"
        actualOperationFolder=$(dirname ${matchedPathFilename})
        actualOperationFile=$(basename ${matchedPathFilename})
        echo "full path to operational file [${operationFile}] [${actualOperationFolder}]"
        echo "full name of operational file [${operationFile}] [${actualOperationFile}]"
      else
        errMsg="unsupported action [${operationAction}]"
        exitWithErrMsg "$errMsg"
      fi

      case "$operationAction" in
        delete)
          doDeleteAction $operationDestinationType $actualOperationFile $actualDestinationFolder
          ;;
        copy)
          doCopyAction $operationDestinationType $actualOperationFolder $actualOperationFile $operationDestinationFolder
          ;;
        replace)
          # copy the file to the folder, but first save any existing version
          savedFilesFolder=${savedFilesFolderHead}/$operationDestinationFolderTail
          # ensure there is a "saved files folder"
          makeFolderFullPath ${savedFilesFolder}
          doReplaceAction $operationDestinationType $actualOperationFolder $actualOperationFile $operationDestinationFolder $savedFilesFolder
          ;;
          
        backup)
          # save the existing version
          savedFilesFolder=${savedFilesFolderHead}/$actualDestinationFolderTail
          # ensure there is a "saved files folder"
          makeFolderFullPath ${savedFilesFolder}
          doBackupAction $operationDestinationType $actualOperationFile $actualDestinationFolder $savedFilesFolder
          ;;
          
        backupDelete)
          # save then delete any existing version
          savedFilesFolder=${savedFilesFolderHead}/$actualDestinationFolderTail
          # ensure there is a "saved files folder"
          makeFolderFullPath ${savedFilesFolder}
          doBackupAction $operationDestinationType $actualOperationFile $actualDestinationFolder $savedFilesFolder
          doDeleteAction $operationDestinationType $actualOperationFile $actualDestinationFolder
          ;;
          
        unzip)
          # extract the contents of the zip file direct to the destination folder
          echo "==== extracting contents of zip file [$actualOperationFile] direct to folder [$operationDestinationFolder] ===="
          echo "     using [-o] option to force overwriting of any existing files"
          # ensure there is a "target folder"
          makeFolderFullPath ${operationDestinationFolder}
          cd $actualOperationFolder
          unzipZip $actualOperationFile ${operationDestinationFolder}
          echo "current contents of [${operationDestinationFolder}]"
          ls -la ${operationDestinationFolder}
          ;;
          
        unzipSameDir)
          # extract the contents of the zip file direct into the same folder
          echo "==== extracting contents of zip file [$actualOperationFile] into the same folder [$actualOperationFolder] ===="
          echo "     using [-o] option to force overwriting of any existing files"
          cd $actualOperationFolder
          unzipZip $actualOperationFile ${actualOperationFolder}
          echo "current contents of [${actualOperationFolder}]"
          ls -la ${actualOperationFolder}
          ;;
          
        unzipCopy)
          # extract the contents of the zip file
          tempUnzipFolder=${localIFixZipFolderFiles}"/Temp/"$operationDestinationFolderTail
          makeFolderFullPath ${tempUnzipFolder}
          echo "==== extracting contents of zip file [$actualOperationFile] to temp folder [$tempUnzipFolder] ===="
          echo "     using [-o] option to force overwriting of any existing files"
          cd $actualOperationFolder
          unzipZip $actualOperationFile ${tempUnzipFolder}
          echo "current contents of [${tempUnzipFolder}]"
          ls -la ${tempUnzipFolder}
          
          # assuming that $actualOperationFile is not needed for this one
          doExtractCopyAction $operationDestinationType $tempUnzipFolder $operationDestinationFolder
          ;;
          
        unzipReplace)
          # copy the file to the folder, but first save any existing version
          savedFilesFolder=${savedFilesFolderHead}/$operationDestinationFolderTail
          # ensure there is a "saved files folder"
          makeFolderFullPath ${savedFilesFolder}
          
          # extract the contents of the zip file
          tempUnzipFolder=${localIFixZipFolderFiles}"/Temp/"$operationDestinationFolderTail
          makeFolderFullPath ${tempUnzipFolder}
          echo "==== extracting contents of zip file [$actualOperationFile] to temp folder [$tempUnzipFolder] ===="
          echo "     using [-o] option to force overwriting of any existing files"
          cd $actualOperationFolder
          unzipZip $actualOperationFile ${tempUnzipFolder}
          echo "current contents of [${tempUnzipFolder}]"
          ls -la ${tempUnzipFolder}
          doExtractReplaceAction $operationDestinationType $tempUnzipFolder $operationDestinationFolder $savedFilesFolder
          ;;
          
        untar)
          # extract the contents of the tar file direct to the destination folder
          echo "==== extracting contents of tar file [$actualOperationFile] direct to folder [$operationDestinationFolder] ===="
          echo "     using default tar extract settings, so overwriting any existing files"
          # ensure there is a "target folder"
          makeFolderFullPath ${operationDestinationFolder}
          cd $actualOperationFolder
          untarTar $actualOperationFile ${operationDestinationFolder}
          echo "current contents of [${operationDestinationFolder}]"
          ls -la ${operationDestinationFolder}
          ;;
          
        untarSameDir)
          # extract the contents of the tar file direct into the same folder
          echo "==== extracting contents of tar file [$actualOperationFile] into the same folder [$actualOperationFolder] ===="
          echo "     using default tar extract settings, so overwriting any existing files"
          cd $actualOperationFolder
          untarTar $actualOperationFile ${actualOperationFolder}
          echo "current contents of [${actualOperationFolder}]"
          ls -la ${actualOperationFolder}
          ;;
          
        untarCopy)
          # extract the contents of the tar file
          tempUnzipFolder=${localIFixZipFolderFiles}"/Temp/"$operationDestinationFolderTail
          makeFolderFullPath ${tempUnzipFolder}
          echo "==== extracting contents of tar file [$actualOperationFile] to temp folder [$tempUnzipFolder] ===="
          echo "     using default tar extract settings, so overwriting any existing files"
          cd $actualOperationFolder
          untarTar $actualOperationFile ${tempUnzipFolder}
          echo "current contents of [${tempUnzipFolder}]"
          ls -la ${tempUnzipFolder}
          
          # assuming that $actualOperationFile is not needed for this one
          doExtractCopyAction $operationDestinationType $tempUnzipFolder $operationDestinationFolder
          ;;
          
        untarReplace)
          # copy the file to the folder, but first save any existing version
          savedFilesFolder=${savedFilesFolderHead}/$operationDestinationFolderTail
          # ensure there is a "saved files folder"
          makeFolderFullPath ${savedFilesFolder}
          
          # extract the contents of the tar file
          tempUnzipFolder=${localIFixZipFolderFiles}"/Temp/"$operationDestinationFolderTail
          makeFolderFullPath ${tempUnzipFolder}
          echo "==== extracting contents of tar file [$actualOperationFile] to temp folder [$tempUnzipFolder] ===="
          echo "     using default tar extract settings, so overwriting any existing files"
          cd $actualOperationFolder
          untarTar $actualOperationFile ${tempUnzipFolder}
          echo "current contents of [${tempUnzipFolder}]"
          ls -la ${tempUnzipFolder}
          doExtractReplaceAction $operationDestinationType $tempUnzipFolder $operationDestinationFolder $savedFilesFolder
          ;;
          
          
        modifyTspKey)
          # update or add a key=value pair to a teamserver.properties file
          # $operationMiscString contains the key=value pair
          # but first save any existing version
          savedFilesFolder=${savedFilesFolderHead}/$actualDestinationFolderTail
          # ensure there is a "saved files folder"
          makeFolderFullPath ${savedFilesFolder}
          doModifyTspKey $operationMiscString $actualOperationFile $actualDestinationFolder $savedFilesFolder
          ;;
          
        deleteTspKey)
          # delete a key from a teamserver.properties file
          # $operationMiscString contains the key
          # but first save any existing version
          savedFilesFolder=${savedFilesFolderHead}/$actualDestinationFolderTail
          # ensure there is a "saved files folder"
          makeFolderFullPath ${savedFilesFolder}
          doDeleteTspKey $operationMiscString $actualOperationFile $actualDestinationFolder $savedFilesFolder
          ;;
          
       *)
          errMsg="unsupported action [${operationAction}]"
          exitWithErrMsg "$errMsg"
          ;;
      esac
    fi

  fi

done < "$filename"


## =======================================================
## === finished ====
echo "------------------------------------------------------"
errMsg="reached the end of the script"
exitWithOkMsg "$errMsg"
