#!/bin/sh
#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2015. All Rights Reserved.
#
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************

# ====== limits.conf =======
# ==========================

nprochardlimit="10000"
nprocsoftlimit="10000"
nofilehardlimit="65536"
nofilesoftlimit="65536"
corehardlimit="unlimited"
coresoftlimit="unlimited"
stackhardlimit="32768"
stacksoftlimit="32768"

nprochard="hard nproc $nprochardlimit"
nprocsoft="soft nproc $nprocsoftlimit"
nofilehard="hard nofile $nofilehardlimit"
nofilesoft="soft nofile $nofilesoftlimit"
corehard="hard core $corehardlimit"
coresoft="soft core $coresoftlimit"
stackhard="hard stack $stackhardlimit"
stacksoft="soft stack $stacksoftlimit"

foundnprochard=FALSE
foundnprocsoft=FALSE
foundnofilehard=FALSE
foundnofilesoft=FALSE
foundcorehard=FALSE
foundcoresoft=FALSE
foundstackhard=FALSE
foundstacksoft=FALSE


# ====== set max process for this (root) session =======
# ======================================================
profileFile=/etc/profile
echo "ulimit -u $nprochardlimit" >> $profileFile


# ====== set properties for future (all users) sessions =======
# =============================================================

targetFile=/etc/security/limits.conf
tempFile=temp_limits.conf

cp $targetFile $targetFile.BAK
if [ -f $tempFile ]; then
  rm -f $tempFile
fi
touch $tempFile

echo "updating [$targetFile]"
while read currentline
do
  
  #END OF FILE (last line)
  if [[ $currentline == *End*of*file* ]]; then

    if [[ $foundnprochard == "FALSE" ]]; then
      echo "[nprochard] line never found, so need to add it"
      echo "* ${nprochard}" >> $tempFile    
    fi
    if [[ $foundnprocsoft == "FALSE" ]]; then
      echo "[nprocsoft] line never found, so need to add it"
      echo "* ${nprocsoft}" >> $tempFile
    fi

    if [[ $foundnofilehard == "FALSE" ]]; then
      echo "[nofilehard] line never found, so need to add it"
      echo "* ${nofilehard}" >> $tempFile   
    fi
    if [[ $foundnofilesoft == "FALSE" ]]; then
      echo "[nofilesoft] line never found, so need to add it"
      echo "* ${nofilesoft}" >> $tempFile
    fi
    if [[ $foundcorehard == "FALSE" ]]; then
      echo "[corehard] line never found, so need to add it"
      echo "* ${corehard}" >> $tempFile
    fi
    if [[ $foundcoresoft == "FALSE" ]]; then
      echo "[coresoft] line never found, so need to add it"
      echo "* ${coresoft}" >> $tempFile
    fi    
    if [[ $foundstackhard == "FALSE" ]]; then
      echo "[stackhard] line never found, so need to add it"
      echo "* ${stackhard}" >> $tempFile
    fi
    if [[ $foundstacksoft == "FALSE" ]]; then
      echo "[stacksoft] line never found, so need to add it"
      echo "* ${stacksoft}" >> $tempFile
    fi

    ##echo "remember to add last line  [$currentline]"
    echo "${currentline}" >> $tempFile

  
  #COMMENT
  elif [[ $currentline == "#"* ]]; then
    ## echo "comment, just use current line [$currentline]"
    echo "$currentline" >> $tempFile

  # does line contain the right words
  # [yes] does word contain right size
  #   [yes] do nothing
  #   [no] replace with new string
  # [no] do nothing
  
  #NPROC
  elif [[ $currentline == *hard*nproc* ]]; then
    foundnprochard=TRUE
    if [[ $currentline == *${nprochardlimit}* ]]; then
      echo "[nprochard] line [$currentline] is a match, so no need to change"
      echo "${currentline}" >> $tempFile
    else
      echo "[nprochard] replace [$currentline] with [* ${nprochard}]"
      echo "* ${nprochard}" >> $tempFile
    fi

  elif [[ $currentline == *soft*nproc* ]]; then
    foundnprocsoft=TRUE
    if [[ $currentline == *${nprocsoftlimit}* ]]; then
      echo "[nprocsoft] line [$currentline] is a match, so no need to change"
      echo "${currentline}" >> $tempFile
    else
      echo "[nprocsoft] replace [$currentline] with [* ${nprocsoft}]"
      echo "* ${nprocsoft}" >> $tempFile
    fi
  
  #NOFILE
  elif [[ $currentline == *hard*nofile* ]]; then
    foundnofilehard=TRUE
    if [[ $currentline == *${nofilehardlimit}* ]]; then
      echo "[nofilehard] line [$currentline] is a match, so no need to change"
      echo "${currentline}" >> $tempFile
    else
      echo "[nofilehard] replace [$currentline] with [* ${nofilehard}]"
      echo "* ${nofilehard}" >> $tempFile
    fi

  elif [[ $currentline == *soft*nofile* ]]; then
    foundnofilesoft=TRUE
    if [[ $currentline == *${nofilesoftlimit}* ]]; then
      echo "[nofilesoft] line [$currentline] is a match, so no need to change"
      echo "${currentline}" >> $tempFile
    else
      echo "[nofilesoft] replace [$currentline] with [* ${nofilesoft}]"
      echo "* ${nofilesoft}" >> $tempFile
    fi

  ##CORE
  elif [[ $currentline == *hard*core* ]]; then
    foundcorehard=TRUE
    if [[ $currentline == *${corehardlimit}* ]]; then
      echo "[corehard] line [$currentline] is a match, so no need to change"
      echo "${currentline}" >> $tempFile
    else
      echo "[corehard] replace [$currentline] with [* ${corehard}]"
      echo "* ${corehard}" >> $tempFile
    fi

  elif [[ $currentline == *soft*core* ]]; then
    foundcoresoft=TRUE
    if [[ $currentline == *${coresoftlimit}* ]]; then
      echo "[coresoft] line [$currentline] is a match, so no need to change"
      echo "${currentline}" >> $tempFile
    else
      echo "[coresoft] replace [$currentline] with [* ${coresoft}]"
      echo "* ${coresoft}" >> $tempFile
    fi

  ##STACK
  elif [[ $currentline == *hard*stack* ]]; then
    foundstackhard=TRUE
    if [[ $currentline == *${stackhardlimit}* ]]; then
      echo "[stackhard] line [$currentline] is a match, so no need to change"
      echo "${currentline}" >> $tempFile
    else
      echo "[stackhard] replace [$currentline] with [* ${stackhard}]"
      echo "* ${stackhard}" >> $tempFile
    fi

  elif [[ $currentline == *soft*stack* ]]; then
    foundstacksoft=TRUE
    if [[ $currentline == *${stacksoftlimit}* ]]; then
      echo "[stacksoft] line [$currentline] is a match, so no need to change"
      echo "${currentline}" >> $tempFile
    else
      echo "[stacksoft] replace [$currentline] with [* ${stacksoft}]"
      echo "* ${stacksoft}" >> $tempFile
    fi
    

  # OTHER LINES
  else
    ## echo "no match, just use current line [$currentline]"
    echo "$currentline" >> $tempFile
  fi

done < $targetFile

# make the update permananet
cp -f $tempFile $targetFile
rm -f $tempFile 


# ====== 90/20-nproc.conf =======
# ===============================

# targetFile is OS dependant
RHversion=`cat /etc/redhat-release`
##echo "Current Red Hat Release $RHversion"

echo $RHversion | grep "Maipo"
if [ $? -eq 0 ]; then
  echo "OS is Red Hat 7"
  targetFile=/etc/security/limits.d/20-nproc.conf
fi
echo $RHversion | grep "Santiago"
if [ $? -eq 0 ]; then
  echo "OS is Red Hat 6"
  targetFile=/etc/security/limits.d/90-nproc.conf
fi
tempFile=temp_nproc.conf

# reuse existing values for [$nprocsoftlimit] [$nprocsoft]
# reset the line found flag
foundnprocsoft="FALSE"

cp $targetFile $targetFile.BAK
if [ -f $tempFile ]; then
  rm -f $tempFile
fi
touch $tempFile
echo "updating [$targetFile]"
# parse the current file
while read currentline
do
  
  #COMMENT
  if [[ $currentline == "#"* ]]; then
    ## echo "comment, just use current line [$currentline]"
    echo "$currentline" >> $tempFile

  #OWNED BY ROOT
  elif [[ $currentline == "root"* ]]; then
    ## echo "owned by root, just use current line [$currentline]"
    echo "$currentline" >> $tempFile

  # does line contain the right words
  # [yes] does word contain right size
  #   [yes] do nothing
  #   [no] replace with new string
  # [no] do nothing
  
  #NPROC
  elif [[ $currentline == *soft*nproc* ]]; then
    foundnprocsoft=TRUE
    if [[ $currentline == *${nprocsoftlimit}* ]]; then
      echo "[nprocsoft] line [$currentline] is a match, so no need to change"
      echo "${currentline}" >> $tempFile
    else
      echo "[nprocsoft] replace [$currentline] with [* ${nprocsoft}]"
      echo "* ${nprocsoft}" >> $tempFile
    fi

  # OTHER LINES
  else
    ## echo "no match, just use current line [$currentline]"
    echo "$currentline" >> $tempFile
  fi

done < $targetFile

# may still need to add the entry
if [[ $foundnprocsoft == "FALSE" ]]; then
  echo "[nprocsoft] line never found, so need to add it"
  echo "* ${nprocsoft}" >> $tempFile
fi

##echo "remember to add last line  [$currentline]"
echo "${currentline}" >> $tempFile

# make the update permananent
cp -f $tempFile $targetFile
rm -f $tempFile


# ==== the end ====
# =================

echo "update completed"
exit 0 
