#!/bin/sh
#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2015. All Rights Reserved.
#
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************

# expect UCD to supply these scripts in the same directory as <installIM.sh>
# ---------------------------------------------------------------------------------
# script.properties 
# -----------------

#############
## FUNCTIONS 
#############


exitWithErrMsg()
{
  tempString=$1
  echo "-- ERROR [${scriptName}] --"
  echo "-- message: $tempString"
  exit 1
}


## usage checkFileExists tempFile
checkFileExists()
{
  tempFile=$1
  if [ -f $tempFile ]; then
    echo "file [$tempFile] located"
  else
    echo "ERROR: [$tempFile] does not exist"
    exit 1
  fi
}


downloadHttpZip()
{
  fileUrl=$1
  filename=$(basename $fileUrl)

  sh $AUTO_PREP/MediaParameters/doWget.sh $fileUrl
  if [ $? == 0 ]
  then
    echo "download complete"
    checkFileExists $filename
  else
    errMsg="failed to download file ["${filename}"]"
    exitWithErrMsg "$errMsg"
  fi
}

copyZip()
{
  file=$1
  filename=$(basename $file)
  echo "copy ["${file}"] to ["${filename}"]"
  cp $file $filename
  if [ $? == 0 ]
  then
    echo "copy complete" 
    checkFileExists $filename
  else
    errMsg="failed to copy file ["${filename}"]"
    exitWithErrMsg "$errMsg"
  fi
}


unzipZip()
{
  zipFile=$1
  targetDir=$2
  
  unzip ${zipFile} -d ${targetDir}
  if [ $? == 0 ]; then
    echo "unzip successful" 
  else
    errMsg="failed to unzip  ["${zipFile}"]"
    exitWithErrMsg "$errMsg"
  fi
}


####################
##                ##
## ==== MAIN ==== ##
##                ##
####################

timestamp=`date +%s`
echo "DEBUG (start) : current epoch time is ["$timestamp"] Sec"



#
#imVersion=$1

# set up local environment for this script
scriptName=$0
workingDir=`pwd -P`
envProp=/etc/env.properties
scriptProp=./script.properties

. $envProp
. $scriptProp


# get credentials
userCreds=$AUTO_PREP/UserParameters/userCreds.properties
. $userCreds

# get media properties
mediaProp=$AUTO_PREP/MediaParameters/media.properties
. $mediaProp



# Set Linux 64 bit install image name
# ===================================
## imFile=agent.installer.linux.gtk.x86_64.zip
IM_templateFile=IM_response_template.xml
IMXML=$workingDir/IMresponses.xml


if [ "ZIP" == $imMediaType ] || [ "FILE" == $imMediaType ]; then 
  echo "using [${imMediaType}] media location type"
else
  errMsg="["${imMediaType}"] media location type not supported"
  exitWithErrMsg "$errMsg"
fi


if [ "X"$nfsEnabled"X" == "XX" ]; then
  echo "no NFS drive exists, assuming any ZIP files are local"
  nfsEnabled=FALSE
else
  # need to check value
  if [ "FALSE" == $nfsEnabled ]; then
    echo "no NFS drive exists, assuming any ZIP files are local"
    nfsEnabled=FALSE
  elif [ "TRUE" == $nfsEnabled ]; then 
    echo "NFS drive exists, assuming any ZIP files are remote"
  else
    errMsg="["${nfsEnabled}"] nfsEnabled not supported"
    exitWithErrMsg "$errMsg"
  fi
fi

if [ "X"$createMediaOnly"X" == "XX" ]; then
  echo "installing Installation Manager"
  createMediaOnly=FALSE
else
  # need to check value
  if [ "FALSE" == $createMediaOnly ]; then
    echo "installing Installation Manager"
    createMediaOnly=FALSE
  elif [ "TRUE" == $createMediaOnly ]; then 
    echo "adding Installation Manager installer to media server"
  else
    errMsg="["${createMediaOnly}"] createMediaOnlyFlag not supported"
    exitWithErrMsg "$errMsg"
  fi
fi


if [ "X"$debugDoNotInstall"X" == "XX" ]; then
  echo "installing Installation Manager"
  debugDoNotInstall=FALSE
else
  # need to check value
  if [ "FALSE" == $debugDoNotInstall ]; then
    echo "installing Installation Manager"
    debugDoNotInstall=FALSE
  elif [ "TRUE" == $debugDoNotInstall ]; then
    echo "DEBUG: will exit prior to installing Installation Manager"
  else
    errMsg="["${debugDoNotInstall}"] debugDoNotInstallFlag not supported"
    exitWithErrMsg "$errMsg"
  fi
fi


## ======================================================================
## set $imInstallFile location
unzipFolderNeedsDeleted=FALSE
zipNeedsDeleted=FALSE
if [ "FILE" == $imMediaType ]; then 
  ## using file ${mediaRoot}/${imMediaRoot}/${imFile}
  if [ "XX" == "X"${mediaRoot}"X" ] || [ "XX" == "X"${imMediaRoot}"X" ] || [ "XX" == "X"${imFile}"X" ]; then
    errMsg="$!mediaRoot [$mediaRoot] or $!imMediaRoot [$imMediaRoot] or $!imFile [$imFile] not set!"
    exitWithErrMsg "$errMsg"
  else
    imInstallFolder=${mediaRoot}"/"${imMediaRoot}
    imInstallFile=${imFile}
    imInstallFileLocation=${imInstallFolder}"/"${imInstallFile}
    echo "using file $!imInstallFileLocation [$imInstallFileLocation]"
    checkFileExists ${imInstallFileLocation}
  fi
fi
if [ "ZIP" == $imMediaType ]; then
  # need to calculate $imZipFileLocation
  
  if [ "FILE" == $mediaAccess ]; then
    
    ## using file ${mediaRoot}/${imMediaRoot}/${imFile}
    if [ "XX" == "X"${mediaRoot}"X" ] || [ "XX" == "X"${imMediaRoot}"X" ] || [ "XX" == "X"${imFile}"X" ]; then
      errMsg="$!mediaRoot [$mediaRoot] or $!imMediaRoot [$imMediaRoot] or $!imFile [$imFile] not set!"
      exitWithErrMsg "$errMsg"
    else
      imZipFileLocation=${mediaRoot}"/"${imMediaRoot}"/"${imFile}
      checkFileExists ${imZipFileLocation}
    fi
  fi
  if [ "HTTP" == $mediaAccess ] || [ "HTTPS" == $mediaAccess ]; then
    if [ "X"$imFile"X" == "XX" ]; then
      tempStr="notURL"
    else
      tempStr=`echo ${imFile} | cut -c -4`
    fi
    if [ "http" == $tempStr ]; then
      ## $imFile conatins the full URL
      if [ "XX" == "X"${imFile}"X" ]; then
        errMsg="$!imFile [$imFile] not set!"
        exitWithErrMsg "$errMsg"
      else
        imZipFileLocation=${imFile}
      fi
    else
      # calculate the URL = ${mediaServer}/${imMediaRoot}/${imVersion}/${imFile}
      if [ "XX" == "X"${mediaServer}"X" ] || [ "XX" == "X"${imMediaRoot}"X" ] || [ "XX" == "X"${imVersion}"X" ] || [ "XX" == "X"${imFile}"X" ]; then
        errMsg="$!mediaServer [$mediaServer] or $!imMediaRoot [$imMediaRoot] or $!imVersion [$imVersion] not set! or $!imFile [$imFile] not set!"
        exitWithErrMsg "$errMsg"
      else
        imZipFileLocation=${mediaServer}"/"${imMediaRoot}"/"${imVersion}"/"${imFile}
      fi
    fi
  fi  

  ## using zipped up file
  if [ "X"$imZipFileLocation"X" == "XX" ]; then
    errMsg="$!imZipFileLocation [$imZipFileLocation] not set!"
    exitWithErrMsg "$errMsg"
  else
    echo "using zipped up file, $!imZipFileLocation [$imZipFileLocation]"
    zipFileName=$(basename $imZipFileLocation)
    
    ## download the zip (http) OR copy the zip (nfs)
    #downloadDir=${mediaRoot}"/"${imMediaRoot}
    downloadDir=${QD_InternalMedia}"/"${imMediaRoot}
    unzippedDir=${downloadDir}"/unzipped"
    mkdir -p ${downloadDir}
    mkdir -p ${unzippedDir}
    cd ${downloadDir}
    tempStr=`echo ${imZipFileLocation} | cut -c -4`
    timestamp=`date +%s`
    echo "DEBUG (ready to get ZIP) : current epoch time is ["$timestamp"] Sec"
    if [ "http" == $tempStr ]; then
      ## download the zip from the url
      downloadHttpZip $imZipFileLocation
      zipNeedsDeleted=TRUE
    else
      # only download a copy if NFS has been setup
      if [ "TRUE" == ${nfsEnabled} ]; then
        # copy the zip
        copyZip $imZipFileLocation
        zipNeedsDeleted=TRUE
      fi
    fi
    timestamp=`date +%s`
    echo "DEBUG (ready to unzip ZIP) : current epoch time is ["$timestamp"] Sec"
    ## unzip
    if [ "http" == $tempStr ]; then
      unzipZip ${downloadDir}"/"${zipFileName} ${unzippedDir}
      unzipFolderNeedsDeleted=TRUE
    else
      if [ "TRUE" == ${nfsEnabled} ]; then
        unzipZip ${downloadDir}"/"${zipFileName} ${unzippedDir}
        unzipFolderNeedsDeleted=TRUE
      else
        unzipZip $imZipFileLocation ${unzippedDir}
        unzipFolderNeedsDeleted=TRUE
      fi
    fi
    timestamp=`date +%s`
    echo "DEBUG (ZIP unzipped) : current epoch time is ["$timestamp"] Sec"
    
    imInstallFolder=${unzippedDir}
    imInstallFile="userinstc"
    imInstallFileLocation=${imInstallFolder}"/"${imInstallFile}
    echo "using unzipped file, $!imInstallFileLocation [$imInstallFileLocation]"
    checkFileExists ${imInstallFileLocation}
  fi
fi


## =============================================================
## === exit if only creating a media server

if [ "TRUE" == $createMediaOnly ]; then
  echo "media server update done"
  exit 0
fi



## ======================================================================
## ==== create IM install XML ====

cd ${workingDir}
echo "Create IM response file from template [$IM_templateFile]"

if [ -f $IMXML ]; then
  echo "deleting old version of $IMXML"
  rm -f $IMXML
fi
cp $IM_templateFile $IMXML
checkFileExists $IMXML
sed -i s%_IM_PARENT_DIR_%$IMParentDir% $IMXML
sed -i s%_IBMQD_IM_REPO_%$imInstallFolder%g $IMXML


## =============================================================
## === DEBUG early exit

if [ "TRUE" == $debugDoNotInstall ]; then
  echo "DEBUG: Exiting prior to installing installation manager"
  exit 0
fi

# Create local user for the installation
# ======================================
tempPassword=`echo ${OSUserPassword} | base64 -d`

echo "OSUserName: [$OSUserName]"
echo "OSUserPassword: [**XX**]"
echo "OSUserGroup: [$OSUserGroup]"

# Creating group if the group does not exist on the system
getent group $OSUserGroup
if [ $? -eq 0 ]; then
   echo "group [$OSUserGroup] exists"
else
     echo "creating group: [groupadd $OSUserGroup]"
     groupadd $OSUserGroup  
fi

echo "command to create user: [useradd -d /home/$OSUserName -m $OSUserName -g $OSUserGroup]"
id $OSUserName
exitStatus=$?
if [ $exitStatus -eq 0 ];then
  echo user exist, checking if the user belongs to group [$OSUserGroup]
  id $OSUserName | grep $OSUserGroup
  if [ $? -gt 0 ];
  then
     echo "Add user to group [$OSUserGroup]"
     usermod -a -G $OSUserGroup $OSUserName
     if [ $? -gt 0 ];
     then
         echo "ERROR: Failed to add [$OSUserName] to group [$OSUserGroup]"
         exit 1
     fi
  fi
else
  useradd -d /home/$OSUserName -m $OSUserName -g $OSUserGroup
  echo $tempPassword | passwd $OSUserName --stdin
fi


# Run setup 
# =========
# installer is at $imInstallFileLocation=${imInstallFolder}"/"${imInstallFile}
# IMexe=./userinstc

cd ${imInstallFolder}
if [ "$OSUserName" == "root" ]
then
   echo "Installing IBM Installation Manager [$imVersion] as root"
   ./${imInstallFile} input $IMXML -acceptLicense 
   if [ $? = 0 ]
   then
      echo "Installation Manager Successful"
   else
      errMsg="Installation Manager install failed"
      exitWithErrMsg "$errMsg"
   fi
else
   echo "Changing /opt/IBM directory owner|group to be the ${OSUserName} | ${OSUserGroup}"
   chown $OSUserName $IMParentDir
   chgrp $OSUserGroup $IMParentDir
   echo "Installing IBM Installation Manager [$imVersion] as $OSUserName"
   su -c "./${imInstallFile} input $IMXML -acceptLicense " -- $OSUserName
#su -c "./installc --launcher.ini silent-install.ini -acceptLicense -log instIM.log" -- $OSUserName

   if [ $? = 0 ]
   then
      echo "Installation Manager Successful"
   else
      errMsg="Installation Manager install failed"
      exitWithErrMsg "$errMsg"
   fi
fi

# tidy up by deleting anything downloaded or unzipped
if [ "TRUE" == $unzipFolderNeedsDeleted ]; then
  echo "Removing the directory ["${unzippedDir}"]"
  rm -Rf ${unzippedDir}
fi
if [ "TRUE" == $zipNeedsDeleted ]; then
  echo "Removing the directory ["${downloadDir}"]"
  rm -Rf ${downloadDir}
fi

## =======================================================
## === finished ====
timestamp=`date +%s`
echo "DEBUG (EOF) : current epoch time is ["$timestamp"] Sec"
echo "SUCCESS: installation complete"
exit 0