#!/bin/sh
#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2015, 2016. All Rights Reserved.
#
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************
# Name: installWAS.sh
#
# Installs Websphere as either a WAS server or an IHS server
#   Execute the program: sh .\installWAS.sh
#********************************************************************************

# expect UCD to supply these scripts in the same directory as <installWAS.sh>
# ---------------------------------------------------------------------------------
# responseFile_tempate.rsp



#############
## FUNCTIONS 
#############


exitWithErrMsg()
{
  tempString=$1
  echo "-- ERROR [${scriptName}] --"
  echo "-- message: $tempString"
  exit 1
}


## usage checkFileExists tempFile
checkFileExists()
{
  tempFile=$1
  if [ -f $tempFile ]; then
    echo "file [$tempFile] located"
  else
    errMsg="[$tempFile] does not exist"
    exitWithErrMsg "$errMsg"
  fi
}


downloadHttpZip()
{
  fileUrl=$1
  filename=$(basename $fileUrl)

  sh $AUTO_PREP/MediaParameters/doWget.sh $fileUrl
  if [ $? == 0 ]
  then
    echo "download complete"
    checkFileExists $filename
  else
    errMsg="failed to download file ["${filename}"]"
    exitWithErrMsg "$errMsg"
  fi
}

copyZip()
{
  file=$1
  filename=$(basename $file)
  echo "copy ["${file}"] to ["${filename}"]"
  cp $file $filename
  if [ $? == 0 ]
  then
    echo "copy complete" 
    checkFileExists $filename
  else
    errMsg="failed to copy file ["${filename}"]"
    exitWithErrMsg "$errMsg"
  fi
}


unzipZip()
{
  zipFile=$1
  targetDir=$2
  
  unzip ${zipFile} -d ${targetDir}
  if [ $? == 0 ]; then
    echo "unzip successful" 
  else
    errMsg="failed to unzip  ["${zipFile}"]"
    exitWithErrMsg "$errMsg"
  fi
}


getWasBaseZipFiles()
{
  urlBase=$1
  wasBaseNameHead=$2
  wasBaseDiskCount=$3

  for (( c=1; c<=${wasBaseDiskCount}; c++ )); do
    downloadHttpZip ${urlBase}/${wasBaseNameHead}_${c}_OF_${wasBaseDiskCount}.zip
  done
}


getWasFixPackZipFiles()
{
  urlBase=$1
  wasFPNameHead=$2
  wasFPDiskCount=$3

  for (( c=1; c<=${wasFPDiskCount}; c++ )); do
    downloadHttpZip ${urlBase}/${wasFPNameHead}-part${c}.zip
  done
}
	
getWasJavaZipFiles()
{
  urlBase=$1
  wasJavaNameHead=$2

  downloadHttpZip ${urlBase}/${wasJavaNameHead}.zip
}

getIhsBaseZipFiles()
{
  urlBase=$1
  ihsBaseNameHead=$2
  ihsBaseDiskCount=$3

  for (( c=1; c<=${ihsBaseDiskCount}; c++ )); do
    downloadHttpZip ${urlBase}/${ihsBaseNameHead}_${c}_OF_${ihsBaseDiskCount}.zip
  done
}


getIhsFixPackZipFiles()
{
  urlBase=$1
  ihsFPNameHead=$2
  ihsFPDiskCount=$3

  for (( c=1; c<=${ihsFPDiskCount}; c++ )); do
    downloadHttpZip ${urlBase}/${ihsFPNameHead}-part${c}.zip
  done
}


unzipWasBaseZipFiles()
{
  wasBaseNameHead=$1
  wasBaseDiskCount=$2
  unzippedDir=$3

  for (( c=1; c<=${wasBaseDiskCount}; c++ )); do
    unzipZip $wasBaseRepo/${wasBaseNameHead}_${c}_OF_${wasBaseDiskCount}.zip ${unzippedDir}
  done
}



unzipWasFixPackZipFiles()
{
  wasFPNameHead=$1
  wasFPDiskCount=$2
  unzippedDir=$3

  for (( c=1; c<=${wasFPDiskCount}; c++ )); do
    unzipZip $wasFPRepo/${wasFPNameHead}-part${c}.zip ${unzippedDir}
  done
}

unzipWasJavaZipFiles()
{
  wasJavaNameHead=$1
  unzippedDir=$2

  unzipZip $wasJavaRepo/${wasJavaNameHead}.zip ${unzippedDir}
}

unzipIhsBaseZipFiles()
{
  ihsBaseNameHead=$1
  ihsBaseDiskCount=$2
  unzippedDir=$3

  for (( c=1; c<=${ihsBaseDiskCount}; c++ )); do
    unzipZip $ihsBaseRepo/${ihsBaseNameHead}_${c}_OF_${ihsBaseDiskCount}.zip ${unzippedDir}
  done
}


unzipIhsFixPackZipFiles()
{
  ihsFPNameHead=$1
  ihsFPDiskCount=$2
  unzippedDir=$3

  for (( c=1; c<=${ihsFPDiskCount}; c++ )); do
    unzipZip $ihsFPRepo/${ihsFPNameHead}-part${c}.zip ${unzippedDir}
  done
}


copyWasBaseZipFiles()
{
  fileBase=$1
  wasBaseNameHead=$2
  wasBaseDiskCount=$3

  for (( c=1; c<=${wasBaseDiskCount}; c++ )); do
    copyZip ${fileBase}/${wasBaseNameHead}_${c}_OF_${wasBaseDiskCount}.zip
    
  done
}


copyWasFixPackZipFiles()
{
  fileBase=$1
  wasFPNameHead=$2
  wasFPDiskCount=$3

  for (( c=1; c<=${wasFPDiskCount}; c++ )); do
    copyZip ${fileBase}/${wasFPNameHead}-part${c}.zip
  done
}

copyWasJavaZipFiles()
{
  fileBase=$1
  wasJavaNameHead=$2

  copyZip ${fileBase}/${wasJavaNameHead}.zip
}

copyIhsBaseZipFiles()
{
  fileBase=$1
  ihsBaseNameHead=$2
  ihsBaseDiskCount=$3

  for (( c=1; c<=${ihsBaseDiskCount}; c++ )); do
    copyZip ${fileBase}/${ihsBaseNameHead}_${c}_OF_${ihsBaseDiskCount}.zip
  done
}


copyIhsFixPackZipFiles()
{
  fileBase=$1
  ihsFPNameHead=$2
  ihsFPDiskCount=$3

  for (( c=1; c<=${ihsFPDiskCount}; c++ )); do
    copyZip ${fileBase}/${ihsFPNameHead}-part${c}.zip
  done
}

checkOS()
{
	####################################
	# may need to check OS 
	bRH6flag=FALSE
	bRH7flag=FALSE
	
	RHversion=`cat /etc/redhat-release`
	echo "Current Red Hat Release $RHversion"
	
	echo $RHversion | grep "Maipo"
	if [ $? -eq 0 ]; then
	  echo "OS is Red Hat 7"
	  bRH7flag=TRUE
	fi
	echo $RHversion | grep "Santiago"
	if [ $? -eq 0 ]; then
	  echo "OS is Red Hat 6"
	  bRH6flag=TRUE
	fi
}
####################
##                ##
## ==== MAIN ==== ##
##                ##
####################

timestamp=`date +%s`
echo "DEBUG [start] : current epoch time is ["$timestamp"] Sec"


# set up local environment for this script
scriptName=$0
workingDir=`pwd -P`
envProp=/etc/env.properties
scriptProp=./script.properties
. $envProp
. $scriptProp
echo "" >> $envProp
cat $scriptProp >> $envProp

# get media properties
mediaProp=$AUTO_PREP/MediaParameters/media.properties
. $mediaProp


if [ "ihs" == "$WASRole" ] || [ "was" == "$WASRole" ]; then
  echo "using [${WASRole}] was role"
else
  errMsg="["${WASRole}"] was role supported"
  exitWithErrMsg "$errMsg"
fi


if [ "REPO" == $wasMediaType ] || [ "ZIP" == $wasMediaType ] || [ "FILE" == $wasMediaType ]; then 
  echo "using [${wasMediaType}] media location type"
else
  errMsg="["${wasMediaType}"] media location type not supported"
  exitWithErrMsg "$errMsg"
fi

#Set in env.properties during Prepare System
if [ "default" != $javaSDKName ] && [ "1.7.1_64" != $javaSDKName ]; then 
  errMsg="invalid Java SDK requested [${javaSDKName}]"
  exitWithErrMsg "$errMsg"
fi

if [ "1.7.1_64" == $javaSDKName ]; then 
	javaVersion="Java7.1"
    wasJavaNameHead=7.1.3.40-WS-IBMWASJAVA-Linux
	javaNoDotVersion=v71
	javaDotVersion=7.1
fi

if [ "X"$nfsEnabled"X" == "XX" ]; then
  echo "no NFS drive exists, assuming zip files are local"
  nfsEnabled=FALSE
else
  # need to check value
  if [ "FALSE" == $nfsEnabled ]; then
    echo "no NFS drive exists, assuming zip files are local"
    nfsEnabled=FALSE
  elif [ "TRUE" == $nfsEnabled ]; then 
    echo "NFS drive exists, assuming zip files are remote"
  else
    errMsg="["${nfsEnabled}"] nfsEnabled not supported"
    exitWithErrMsg "$errMsg"
  fi
fi

if [ "X"$createMediaOnly"X" == "XX" ]; then
  echo "installing DB2 license"
  createMediaOnly=FALSE
else
  # need to check value
  if [ "FALSE" == $createMediaOnly ]; then
    echo "installing "${WASRole}
    createMediaOnly=FALSE
  elif [ "TRUE" == $createMediaOnly ]; then 
    echo "adding "${WASRole}" to media server"
  else
    errMsg="["${createMediaOnly}"] createMediaOnlyFlag not supported"
    exitWithErrMsg "$errMsg"
  fi
fi


if [ "X"$debugDoNotInstall"X" == "XX" ]; then
  echo "installing "${WASRole}
  debugDoNotInstall=FALSE
else
  # need to check value
  if [ "FALSE" == $debugDoNotInstall ]; then
    echo "installing "${WASRole}
    debugDoNotInstall=FALSE
  elif [ "TRUE" == $debugDoNotInstall ]; then
    echo "DEBUG: will exit prior to installing WAS"
  else
    errMsg="["${debugDoNotInstall}"] debugDoNotInstallFlag not supported"
    exitWithErrMsg "$errMsg"
  fi
fi


## ======================================================================
## ==== IM command location, not supporting 163!
IM_LOCATION=/opt/IBM/InstallationManager/eclipse


# 
# set version strings
# =========================
WASBaseVersion=`echo $wasVersion | awk -F. '{$NF="";print}' OFS=.|rev |cut -c 2- |rev`
# Get fixpack version, like 4
FPVersion=`echo $wasVersion |rev | cut -c 1`
# Get 8.5
WASMajorDotVersion=`echo $WASBaseVersion | cut -c 1-3`
# Get 85
WASMajorVersion=`echo $WASMajorDotVersion | cut -c 1,3`

echo "*** WASRole: [$WASRole]"
echo "*** WASBaseVersion: [$WASBaseVersion]"
echo "*** WASVersion: [$wasVersion]"
echo "*** WASMajorDotVersion: [$WASMajorDotVersion]"
echo "*** FPVersion: [$FPVersion]"
echo "*** Java SDK name: [$javaSDKName]"


## ======================================================================
## set repositiory locations
wasZipNeedsDeleted=FALSE
wasUnzipFolderNeedsDeleted=FALSE
ihsZipNeedsDeleted=FALSE
ihsUnzipFolderNeedsDeleted=FALSE
if [ "REPO" == $wasMediaType ]; then 
  ## using generic repos
  if [ "$WASRole" == "was" ]; then  
    if [ "X"$wasBaseRepoFile"X" == "XX" ] || [ "X"$wasFPRepoFile"X" == "XX" ]; then
      errMsg="$!wasBaseRepoFile [$wasBaseRepoFile] or $!wasFPRepoFile [$wasFPRepoFile] not set!"
      exitWithErrMsg "$errMsg"
    else
      wasBaseRepository=$wasBaseRepoFile
      wasFPRepository=$wasFPRepoFile
      echo "using generic repository, $!wasBaseRepository [$wasBaseRepository]"
      echo "using generic repository, $!wasFPRepository [$wasFPRepository]"
    fi
  fi

  if [ "$WASRole" == "ihs" ]; then  
    if [ "X"$ihsBaseRepoFile"X" == "XX" ] || [ "X"$ihsFPRepoFile"X" == "XX" ]; then
      errMsg="$!ihsBaseRepoFile [$ihsBaseRepoFile] or $!ihsFPRepoFile [$ihsFPRepoFile] not set!"
      exitWithErrMsg "$errMsg"
    else
      ihsBaseRepository=$ihsBaseRepoFile
      ihsFPRepository=$ihsFPRepoFile
      echo "using generic repository, $!ihsBaseRepository [$ihsBaseRepository]"
      echo "using generic repository, $!ihsFPRepository [$ihsFPRepository]"
    fi
  fi
fi

if [ "FILE" == $wasMediaType ]; then 
  ## using file repo ${mediaRoot}/${wasMediaRoot}/${wasBaseRepoFile} || {wasFPRepoFile} || {ihsBaseRepoFile} || {ihsFPRepoFile}
  
  if [ "$WASRole" == "was" ]; then  
    if [ "X"$mediaRoot"X" == "XX" ] || [ "X"$wasMediaRoot"X" == "XX" ] || [ "X"$wasBaseRepoFile"X" == "XX" ] || [ "X"$wasFPRepoFile"X" == "XX" ]; then
      echo "$!mediaRoot [$mediaRoot] not set!"
      echo "$!wasMediaRoot [$wasMediaRoot] not set!"
      echo "$!wasBaseRepoFile [$wasBaseRepoFile] not set!"
      echo "$!wasFPRepoFile [$wasFPRepoFile] not set!"
      errMsg="one of the above not set!"
      exitWithErrMsg "$errMsg"
    else
      wasBaseRepository="file:"${mediaRoot}"/"${wasMediaRoot}"/"${wasBaseRepoFile}
      wasFPRepository="file:"${mediaRoot}"/"${wasMediaRoot}"/"${wasFPRepoFile}
      echo "using file repository, $!wasBaseRepository [$wasBaseRepository]"
      echo "using file repository, $!wasFPRepository [$wasFPRepository]"
    fi
  fi

  if [ "$WASRole" == "ihs" ]; then  
    if [ "X"$mediaRoot"X" == "XX" ] || [ "X"$wasMediaRoot"X" == "XX" ] || [ "X"$ihsBaseRepoFile"X" == "XX" ] || [ "X"$ihsFPRepoFile"X" == "XX" ]; then
      echo "$!mediaRoot [$mediaRoot] not set!"
      echo "$!wasMediaRoot [$wasMediaRoot] not set!"
      echo "$!ihsBaseRepoFile [$ihsBaseRepoFile] not set!"
      echo "$!ihsFPRepoFile [$ihsFPRepoFile] not set!"    
      errMsg="one of the above not set!"
      exitWithErrMsg "$errMsg"
    else
      ihsBaseRepository="file:"${mediaRoot}"/"${wasMediaRoot}"/"${ihsBaseRepoFile}
      ihsFPRepository="file:"${mediaRoot}"/"${wasMediaRoot}"/"${ihsFPRepoFile}
      echo "using file repository, $!ihsBaseRepository [$ihsBaseRepository]"
      echo "using file repository, $!ihsFPRepository [$ihsFPRepository]"
    fi
  fi
fi

if [ "ZIP" == $wasMediaType ]; then
  # more than one zip file from each repo so only need to calculate the FOLDER NAME
  # WAS and IHS share the same folder
  if [ "FILE" == $mediaAccess ]; then
    ## using folder ${mediaRoot}/${wasMediaRoot}
    if [ "X"$mediaRoot"X" == "XX" ] || [ "X"$wasMediaRoot"X" == "XX" ]; then
      errMsg="one of $!mediaRoot [$mediaRoot] or $!wasMediaRoot [$wasMediaRoot] not set!"
      exitWithErrMsg "$errMsg"
    else
      wasBaseRepoZipFolder=${mediaRoot}"/"${wasMediaRoot}"/"${WASBaseVersion}
      wasFPRepoZipFolder=${mediaRoot}"/"${wasMediaRoot}"/"${wasVersion}
	  if [ "default" != $javaSDKName ]; then 
	    wasJavaRepoZipFolder=${mediaRoot}"/"${wasMediaRoot}"/"${javaVersion}
	  fi
      ihsBaseRepoZipFolder=$wasBaseRepoZipFolder
      ihsFPRepoZipFolder=$wasFPRepoZipFolder
    fi
  fi

  if [ "HTTP" == $mediaAccess ] || [ "HTTPS" == $mediaAccess ]; then
    ## using full path ${wasRepoFolder} -- http(s)
    ## or ${mediaServer}/${wasMediaRoot} -- file

    if [ "X"$wasRepoFolder"X" == "XX" ]; then
      tempStr="notURL"
    else
      tempStr=`echo ${wasRepoFolder} | cut -c -4`
    fi
    if [ "http" == $tempStr ]; then
      ## $wasRepoFolder conatins the full URL
      wasBaseRepoZipFolder=${wasRepoFolder}"/"${WASBaseVersion}
      wasFPRepoZipFolder=${wasRepoFolder}"/"${wasVersion}
	  if [ "default" != $javaSDKName ]; then 
	    wasJavaRepoZipFolder=${wasRepoFolder}"/"${javaVersion}
	  fi
      ihsBaseRepoZipFolder=$wasBaseRepoZipFolder
      ihsFPRepoZipFolder=$wasFPRepoZipFolder
    else
      # calculate the URL
      if [ "X"$mediaServer"X" == "XX" ] || [ "X"$wasMediaRoot"X" == "XX" ]; then
        errMsg="one of $!mediaServer [$mediaServer] or $!wasMediaRoot [$wasMediaRoot] not set!"
        exitWithErrMsg "$errMsg"
      else
        wasBaseRepoZipFolder=${mediaServer}"/"${wasMediaRoot}"/"${WASBaseVersion}
        wasFPRepoZipFolder=${mediaServer}"/"${wasMediaRoot}"/"${wasVersion}
	    if [ "default" != $javaSDKName ]; then 
	      wasJavaRepoZipFolder=${mediaServer}"/"${wasMediaRoot}"/"${javaVersion}
	    fi
        ihsBaseRepoZipFolder=$wasBaseRepoZipFolder
        ihsFPRepoZipFolder=$wasFPRepoZipFolder
      fi
    fi
  fi
  
  echo "using was/ihs base folder, $!wasBaseRepoZipFolder [$wasBaseRepoZipFolder]"
  echo "using was/ihs fixpack folder, $!wasFPRepoZipFolder [$wasFPRepoZipFolder]"
  
  # Prepare the media download directories
  #========================================
  
  #downloadDir=${mediaRoot}"/"${wasMediaRoot}
  downloadDir=${QD_InternalMedia}"/"${wasMediaRoot}
  mkdir -p ${downloadDir}
  
  if [ "$WASRole" == "was" ]; then
    wasBaseDiskCount=3
    wasFPDiskCount=2
    wasBaseNameHead=WAS_V${WASBaseVersion}
    wasFPNameHead=${WASBaseVersion}-WS-WAS-FP000000${FPVersion}
    wasBaseRepo=${downloadDir}/wasBaseRepo
    wasFPRepo=${downloadDir}/wasFPRepo
    wasJavaRepo=${downloadDir}/wasJavaRepo
    wasBaseRepoUnzipped=${wasBaseRepo}/repo
    wasFPRepoUnzipped=${wasFPRepo}/repo
    wasJavaRepoUnzipped=${wasJavaRepo}/repo
    mkdir -p $wasBaseRepo
    mkdir -p $wasFPRepo
    mkdir -p $wasJavaRepo
    mkdir -p $wasBaseRepoUnzipped
    mkdir -p $wasFPRepoUnzipped
    mkdir -p $wasJavaRepoUnzipped
  fi
  if [ "$WASRole" == "ihs" ]; then
    ihsBaseDiskCount=3
    ihsFPDiskCount=2
    ihsBaseNameHead=WAS_V${WASBaseVersion}_SUPPL
    ihsFPNameHead=${WASBaseVersion}-WS-WASSupplements-FP000000${FPVersion}
    ihsBaseRepo=${downloadDir}/ihsBaseRepo
    ihsFPRepo=${downloadDir}/ihsFPRepo
    ihsBaseRepoUnzipped=${ihsBaseRepo}/repo
    ihsFPRepoUnzipped=${ihsFPRepo}/repo
    mkdir -p $ihsBaseRepoUnzipped
    mkdir -p $ihsFPRepoUnzipped
    mkdir -p $ihsBaseRepo
    mkdir -p $ihsFPRepo
  fi


  # Copy install media 
  # ==================
  echo "Get $WASRole Media..."

  timestamp=`date +%s`
  echo "DEBUG (ready to get ZIPs) : current epoch time is ["$timestamp"] Sec"
  if [ "$WASRole" == "was" ]; then
    tempStr=`echo ${wasBaseRepoZipFolder} | cut -c -4`
    if [ "http" == $tempStr ]; then
      ## download the zips from the url
      echo "get base image zips"
      cd $wasBaseRepo
      getWasBaseZipFiles $wasBaseRepoZipFolder $wasBaseNameHead $wasBaseDiskCount
      cd $wasFPRepo
      getWasFixPackZipFiles $wasFPRepoZipFolder $wasFPNameHead $wasFPDiskCount
      wasZipNeedsDeleted=TRUE
      if [ "default" != $javaSDKName ]; then 
        cd $wasJavaRepo
        getWasJavaZipFiles $wasJavaRepoZipFolder $wasJavaNameHead
	  fi
    else
      # only download a copy if NFS has been setup
      if [ "TRUE" == ${nfsEnabled} ]; then
        # copy the zips
        echo "get base image zips"
        cd $wasBaseRepo
        copyWasBaseZipFiles $wasBaseRepoZipFolder $wasBaseNameHead $wasBaseDiskCount
        cd $wasFPRepo
        copyWasFixPackZipFiles $wasFPRepoZipFolder $wasFPNameHead $wasFPDiskCount
        wasZipNeedsDeleted=TRUE
	    if [ "default" != $javaSDKName ]; then 
          cd $wasJavaRepo
          copyWasJavaZipFiles $wasJavaRepoZipFolder $wasJavaNameHead 
		fi
      fi
    fi
  fi
  if [ "$WASRole" == "ihs" ]; then
    tempStr=`echo ${ihsBaseRepoZipFolder} | cut -c -4`
    if [ "http" == $tempStr ]; then
      ## download the zips from the url
      echo "get base image zips"
      cd $ihsBaseRepo
      getIhsBaseZipFiles $ihsBaseRepoZipFolder $ihsBaseNameHead $ihsBaseDiskCount
      cd $ihsFPRepo
      getIhsFixPackZipFiles $ihsFPRepoZipFolder $ihsFPNameHead $ihsFPDiskCount
      ihsZipNeedsDeleted=TRUE
    else
      # only download a copy if NFS has been setup
      if [ "TRUE" == ${nfsEnabled} ]; then
        # copy the zips
        echo "get base image zips"
        cd $ihsBaseRepo
        copyIhsBaseZipFiles $ihsBaseRepoZipFolder $ihsBaseNameHead $ihsBaseDiskCount
        cd $ihsFPRepo
        copyIhsFixPackZipFiles $ihsFPRepoZipFolder $ihsFPNameHead $ihsFPDiskCount
        ihsZipNeedsDeleted=TRUE
      fi
    fi
  fi

  # Unzip install media 
  # ==================
  echo "Unzip $WASRole Media..."
  
  timestamp=`date +%s`
  echo "DEBUG (ready to unzip ZIP) : current epoch time is ["$timestamp"] Sec"
  
  if [ "$WASRole" == "was" ]; then
    if [ "http" == $tempStr ]; then
      echo "unzip the base image zips"
      cd $wasBaseRepo
      unzipWasBaseZipFiles $wasBaseNameHead $wasBaseDiskCount $wasBaseRepoUnzipped
      echo "unzip the fix pack zips"
      cd $wasFPRepo
      unzipWasFixPackZipFiles $wasFPNameHead $wasFPDiskCount $wasFPRepoUnzipped
      wasUnzipFolderNeedsDeleted=TRUE
      if [ "default" != $javaSDKName ]; then 
        echo "unzip Java SDK"
        cd $wasJavaRepo
        unzipWasJavaZipFiles $wasJavaNameHead $wasJavaRepoUnzipped
	  fi
    else
      if [ "TRUE" == ${nfsEnabled} ]; then
        echo "unzip the base image zips"
        cd $wasBaseRepo
        unzipWasBaseZipFiles $wasBaseNameHead $wasBaseDiskCount $wasBaseRepoUnzipped
        echo "unzip the fix pack zips"
        cd $wasFPRepo
        unzipWasFixPackZipFiles $wasFPNameHead $wasFPDiskCount $wasFPRepoUnzipped
        wasUnzipFolderNeedsDeleted=TRUE
	    if [ "default" != $javaSDKName ]; then 
          echo "unzip Java SDK"
          cd $wasJavaRepo
          unzipWasJavaZipFiles $wasJavaNameHead $wasJavaRepoUnzipped
		fi
      else
        echo "unzip the base image zips"
        wasBaseRepo=$wasBaseRepoZipFolder
        cd $wasBaseRepo
        unzipWasBaseZipFiles $wasBaseNameHead $wasBaseDiskCount $wasBaseRepoUnzipped
        echo "unzip the fix pack zips"
        wasFPRepo=$wasFPRepoZipFolder
        cd $wasFPRepo
        unzipWasFixPackZipFiles $wasFPNameHead $wasFPDiskCount $wasFPRepoUnzipped
        wasUnzipFolderNeedsDeleted=TRUE
	    if [ "default" != $javaSDKName ]; then 
          echo "unzip Java SDK"
          wasJavaRepo=$wasJavaRepoZipFolder
          cd $wasJavaRepo
          unzipWasJavaZipFiles $wasJavaNameHead $wasJavaRepoUnzipped
		fi
      fi
    fi
  fi
  if [ "$WASRole" == "ihs" ]; then
    if [ "http" == $tempStr ]; then
      echo "unzip the base image zips"
      cd $ihsBaseRepo
      unzipIhsBaseZipFiles $ihsBaseNameHead $ihsBaseDiskCount $ihsBaseRepoUnzipped
      echo "unzip the fix pack zips"
      cd $ihsFPRepo
      unzipIhsFixPackZipFiles $ihsFPNameHead $ihsFPDiskCount $ihsFPRepoUnzipped
      ihsUnzipFolderNeedsDeleted=TRUE
    else
      if [ "TRUE" == ${nfsEnabled} ]; then
        echo "unzip the base image zips"
        cd $ihsBaseRepo
        unzipIhsBaseZipFiles $ihsBaseNameHead $ihsBaseDiskCount $ihsBaseRepoUnzipped
        echo "unzip the fix pack zips"
        cd $ihsFPRepo
        unzipIhsFixPackZipFiles $ihsFPNameHead $ihsFPDiskCount $ihsFPRepoUnzipped
        ihsUnzipFolderNeedsDeleted=TRUE
      else
        echo "unzip the base image zips"
        ihsBaseRepo=$ihsBaseRepoZipFolder
        cd $ihsBaseRepo
        unzipIhsBaseZipFiles $ihsBaseNameHead $ihsBaseDiskCount $ihsBaseRepoUnzipped
        echo "unzip the fix pack zips"
        ihsFPRepo=$wasFPRepoZipFolder
        cd $ihsFPRepo
        unzipIhsFixPackZipFiles $ihsFPNameHead $ihsFPDiskCount $ihsFPRepoUnzipped
        ihsUnzipFolderNeedsDeleted=TRUE
      fi
    fi
  fi
  
  timestamp=`date +%s`
  echo "DEBUG (ZIP unzipped) : current epoch time is ["$timestamp"] Sec"
  
  
  # set the IM repository locations
  # ====================================
  if [ "$WASRole" == "was" ]; then  
    wasBaseRepository="file:"$wasBaseRepoUnzipped"/repository.config"
    wasFPRepository="file:"$wasFPRepoUnzipped"/repository.config"
    echo "using file repository, $!wasBaseRepository [$wasBaseRepository]"
    echo "using file repository, $!wasFPRepository [$wasFPRepository]"
    if [ "default" != $javaSDKName ]; then 
      wasJavaRepository="file:"$wasJavaRepoUnzipped"/repository.config"
      echo "using file repository, $!wasJavaRepository [$wasJavaRepository]"
	fi
  fi

  if [ "$WASRole" == "ihs" ]; then  
    ihsBaseRepository="file:"$ihsBaseRepoUnzipped"/repository.config"
    ihsFPRepository="file:"$ihsFPRepoUnzipped"/repository.config"
    echo "using file repository, $!ihsBaseRepository [$ihsBaseRepository]"
    echo "using file repository, $!ihsFPRepository [$ihsFPRepository]"
  fi
  
fi


## =============================================================
## === exit if only creating a media server

if [ "TRUE" == $createMediaOnly ]; then
  echo "media server update done"
  exit 0
fi


## ======================================================================
## ==== create IM install XML ====
# go back to the work directory, prepare the response file
cd $workingDir
IM_templateFile=$workingDir/was_response_template.xml
IMXML=$workingDir"/WASResponse.xml"
echo "Create IM response file from template [$IM_templateFile]"

if [ -f $IMXML ]; then
  echo "deleting old version of $IMXML"
  rm -f $IMXML
fi
echo "The install response file: [$IMXML]"
cp $IM_templateFile $IMXML
checkFileExists $IMXML

if [ "$WASRole" == "was" ]; then
  sed -i s%!--_BG_WAS_BASE_REPO_%% $IMXML
  sed -i s%_WAS_BASE_INSTALL_REPO_%$wasBaseRepository% $IMXML
  sed -i s%_END_WAS_BASE_REPO_--%% $IMXML
  
  sed -i s%!--_BG_WAS_FP_REPO_%% $IMXML
  sed -i s%_WAS_FP_INSTALL_REPO_%$wasFPRepository% $IMXML
  sed -i s%_END_WAS_FP_REPO_--%% $IMXML

  if [ "default" != $javaSDKName ]; then 
    sed -i s%!--_BG_WAS_JAVA_REPO_%% $IMXML
    sed -i s%_WAS_JAVA_INSTALL_REPO_%$wasJavaRepository% $IMXML
    sed -i s%_END_WAS_JAVA_REPO_--%% $IMXML
  fi
fi

if [ "$WASRole" == "ihs" ]; then
  sed -i s%!--_BG_IHS_BASE_REPO_%% $IMXML
  sed -i s%_IHS_BASE_INSTALL_REPO_%$ihsBaseRepository% $IMXML
  sed -i s%_END_IHS_BASE_REPO_--%% $IMXML
  
  sed -i s%!--_BG_IHS_FP_REPO_%% $IMXML
  sed -i s%_IHS_FP_INSTALL_REPO_%$ihsFPRepository% $IMXML
  sed -i s%_END_IHS_FP_REPO_--%% $IMXML
fi


sed -i s%_MAJOR_DOT_VER_%$WASMajorDotVersion% $IMXML
sed -i s%_MAJOR_VER_%$WASMajorVersion% $IMXML

if [ "$WASRole" == "was" ]
then
   sed -i s%_WAS_INSTALL_DIR_%$WAS_ROOT% $IMXML
   if [ "default" != $javaSDKName ]; then 
     sed -i s%"<!--_BG_WAS_JAVA_PROFILE_DATA_"%% $IMXML
     sed -i s%"_END_WAS_JAVA_PROFILE_DATA_-->"%% $IMXML
     sed -i s%_JAVA_VER_%$javaNoDotVersion% $IMXML
	 sed -i s%_JAVA_DOT_VER_%$javaDotVersion% $IMXML
   else
     sed -i s%"_END_WAS_PROFILE_DATA_-->"%% $IMXML
     sed -i s%"<!--_BG_WAS_PROFILE_DATA_"%% $IMXML
   fi
elif [ "$WASRole" == "ihs" ]
then
   sed -i s%_HTTP_INSTALL_DIR_%$IHS_ROOT% $IMXML
   sed -i s%_PLUGINS_INSTALL_DIR_%$Plugins_ROOT% $IMXML
   sed -i s%"_END_IHS_PROFILE_DATA_-->"%% $IMXML
   sed -i s%"<!--_BG_IHS_PROFILE_DATA_"%% $IMXML
fi
echo "installing [$WASRole] version [$wasVersion] using [$IMXML]"
checkFileExists $IMXML

## =============================================================
## === DEBUG early exit

if [ "TRUE" == $debugDoNotInstall ]; then
  echo "DEBUG: Exiting prior to installing "${WASRole}
  exit 0
fi


## =============================================================
# Run the installation via installation manager
IM_USERNAME=`ls -l ${IM_LOCATION}/IBMIM | cut -d' ' -f 3`
IM_USER_GROUP=`ls -l ${IM_LOCATION}/IBMIM | cut -d' ' -f 4`
IMexe="IBMIM --launcher.ini silent-install.ini -input $IMXML -acceptLicense"
echo IM command [$IMexe]

cd $IM_LOCATION
su -c "./$IMexe" -- $IM_USERNAME
if [ $? -gt 0 ]; then
  errMsg="[$WASRole] [$wasVersion] install failed"
  exitWithErrMsg "$errMsg"
fi


if [ "$WASRole" == "was" ]; then
  # Create WAS profile
  # ================================
  installProfileFile=$workingDir/profile.xml
  cd $workingDir
  cp $workingDir/profile_linux_template.response $installProfileFile

  sed -i s%_HostName_%$HOSTNAME% $installProfileFile
  sed -i s%_WAS_ROOT_%$WAS_ROOT% $installProfileFile

  echo "Creating the profile using [$installProfileFile]"
  echo "Command: [su -c "$WAS_ROOT/bin/manageprofiles.sh -response $installProfileFile" -- $IM_USERNAME]"
  su -c "$WAS_ROOT/bin/manageprofiles.sh -response $installProfileFile" -- $IM_USERNAME   
    
    # setting up automatic start up scripts
  sed -i s%_WAS_ROOT_%$WAS_ROOT% $workingDir/ibm-was
  sed -i s%_WAS_INSTALL_USER_%$IM_USERNAME% $workingDir/ibm-was
  
  checkOS
  if [ "TRUE" == $bRH6flag ]; then
    echo "*** Setup $workingDir/ibm-was init.d"
    cp $workingDir/ibm-was /etc/init.d
    chmod o+x /etc/init.d/ibm-was
    chkconfig --add ibm-was
    echo "checking start up scripts"
    ls -l /etc/init.d/ibm-was
    chkconfig --list ibm-was
  else
    echo "*** Setup $workingDir/ibm-was systemd"
    if [ ! -d /usr/lib/systemd/scripts/ ];then
      mkdir /usr/lib/systemd/scripts/
    fi
    cp $workingDir/ibm-was /usr/lib/systemd/scripts/
    chmod o+x /usr/lib/systemd/scripts/ibm-was
    cp $workingDir/ibm-was.service /usr/lib/systemd/system/
    systemctl enable ibm-was.service
    echo "checking start up scripts"
    ls -l /usr/lib/systemd/scripts/ibm-was
    ls -la /etc/systemd/system/multi-user.target.wants | grep ibm-was.service
  fi
  # Start the Application server
  #=====================================
  echo "Start WebSphere process as [$IM_USERNAME]"
  su -c "$WAS_PROFILE_ROOT/bin/startServer.sh server1" -- $IM_USERNAME
  if [ $? -eq 0 ]; then
      echo "WAS [$wasVersion] started successfully"
      
  else
    errMsg="WAS [$wasVersion] start failed"
    exitWithErrMsg "$errMsg"
  fi

  # Set Java SDK
  # as of 603 the minimum Java SDK is 7.1
  #=====================================
  if [ "default" != $javaSDKName ]; then 
	echo "*** Set default java sdkname to ${javaSDKName}"
	echo "*** Command: [su -c "$WAS_ROOT/bin/managesdk.sh -enableProfile -profileName AppSrv01 -sdkname ${javaSDKName} -enableServers" -- $IM_USERNAME]"
	su -c "$WAS_ROOT/bin/managesdk.sh -enableProfile -profileName AppSrv01 -sdkname ${javaSDKName} -enableServers" -- $IM_USERNAME   
	if [ $? -ne 0 ]; then
		errMsg="Failed to set default java sdkname to ${javaSDKName}"
		exitWithErrMsg "$errMsg"
	fi
	echo "*** Check default java sdkname"
	su -c "$WAS_ROOT/bin/managesdk.sh -listEnabledProfile -profileName AppSrv01 -verbose" -- $IM_USERNAME   
	if [ $? -ne 0 ]; then
		errMsg="Unable to check default java sdkname"
		exitWithErrMsg "$errMsg"
	fi
  fi
fi

if [ "$WASRole" == "ihs" ]; then
  # setting up automatically start up scripts
  sed -i s%_IHS_HOME_%$IHS_ROOT% $workingDir/ibm-ihs
  
  checkOS
  if [ "TRUE" == $bRH6flag ]; then
    echo "*** Setup $workingDir/ibm-ihs init.d"
    cp $workingDir/ibm-ihs /etc/init.d
    chmod o+x /etc/init.d/ibm-ihs
    chkconfig --add ibm-ihs
    echo "checking start up scripts"
    ls -l /etc/init.d/ibm-ihs
    chkconfig --list ibm-ihs
  else
    echo "*** Setup $workingDir/ibm-ihs systemd"
    if [ ! -d /usr/lib/systemd/scripts/ ];then
      mkdir /usr/lib/systemd/scripts/
    fi
    cp $workingDir/ibm-ihs /usr/lib/systemd/scripts/
    chmod o+x /usr/lib/systemd/scripts/ibm-ihs
    cp $workingDir/ibm-ihs.service /usr/lib/systemd/system/
    systemctl enable ibm-ihs.service
    echo "checking start up scripts"
    ls -l /usr/lib/systemd/scripts/ibm-ihs
    ls -la /etc/systemd/system/multi-user.target.wants | grep ibm-ihs.service
  fi
  
  # Setting up IBM HTTP Server for starting/stopping from WAS console
  # Probably not need to run setupadm if we do not want to run from WAS console to stop the IHS  
  cd $IHS_ROOT/bin
  ./setupadm -usr $IM_USERNAME -grp $IM_USER_GROUP -cfg $IHS_ROOT/conf/httpd.conf -adm $IHS_ROOT/conf/admin.conf
  sed -i s%@@AdminPort@@%8008% $IHS_ROOT/conf/admin.conf

  # Start the HTTP server
  #=====================================
  echo "starting HTTP server process...."
  $IHS_ROOT/bin/adminctl start
  if [ $? == 0 ]; then
      echo " IHS admin process started successfully "
  else
    errMsg="IHS admin process [$IHS_ROOT/bin/adminctl start] failed"
    exitWithErrMsg "$errMsg"
  fi
  
  $IHS_ROOT/bin/apachectl start
  if [ $? == 0 ]; then
      echo " IHS http process started successfully "
  else
    errMsg="IHS http process [$IHS_ROOT/bin/apachectl start] failed"
    exitWithErrMsg "$errMsg"
  fi
fi


# tidy up by deleting anything downloaded or unzipped
# ===================================================
if [ "TRUE" == $wasUnzipFolderNeedsDeleted ]; then
  echo "Removing the WAS base repo directory ["${wasBaseRepoUnzipped}"]"
  rm -Rf ${wasBaseRepoUnzipped}
  echo "Removing the WAS fix pack repo directory ["${wasFPRepoUnzipped}"]"
  rm -Rf ${wasFPRepoUnzipped}
  if [ -d ${wasJavaRepoUnzipped} ]; then
    echo "Removing the WAS java repo directory ["${wasJavaRepoUnzipped}"]"
    rm -Rf ${wasJavaRepoUnzipped}
  fi
fi
if [ "TRUE" == $ihsUnzipFolderNeedsDeleted ]; then
  echo "Removing the IHS base repo directory ["${ihsBaseRepoUnzipped}"]"
  rm -Rf ${ihsBaseRepoUnzipped}
  echo "Removing the IHS fix pack rrepo directory ["${ihsFPRepoUnzipped}"]"
  rm -Rf ${ihsFPRepoUnzipped}
fi
if [ "TRUE" == $wasZipNeedsDeleted ]; then
  echo "Removing the downloaded WAS base zip file directory ["${wasBaseRepo}"]"
  rm -Rf ${wasBaseRepo}
  echo "Removing the downloaded WAS fix pack zip file directory ["${wasFPRepo}"]"
  rm -Rf ${wasFPRepo}
  if [ -d ${wasJavaRepo} ]; then
    echo "Removing the downloaded WAS java zip file directory ["${wasJavaRepo}"]"
    rm -Rf ${wasJavaRepo}
  fi
fi
if [ "TRUE" == $ihsZipNeedsDeleted ]; then
  echo "Removing the downloaded IHS base zip file directory ["${ihsBaseRepo}"]"
  rm -Rf ${ihsBaseRepo}
  echo "Removing the downloaded IHS fix pack zip file directory ["${ihsFPRepo}"]"
  rm -Rf ${ihsFPRepo}
fi


## =======================================================
## === finished ====
timestamp=`date +%s`
echo "DEBUG [EOF] : current epoch time is ["$timestamp"] Sec"
echo "SUCCESS: installation complete"
exit 0