#!/bin/bash
#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2015. All Rights Reserved.
#
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************

##########################
# Expect the argument
# RunSetup=Y/N MTM=Y/N ImportUser=Y/N
###############################


#######################
# ---- Functions ---- #
#######################


## parse all apps and see if DM is present
isDMpresent()
{
  nodeCount=$1
  DMpresentFlag=FALSE
  i=1
  while [ $i -le $nodeCount ]
  do
    c_app_var=NODE${i}_AppList
    eval c_app=\$$c_app_var
    appList=${c_app//,/ }
    for a in $appList
    do
      if [ "dm" == $a  ]; then
        DMpresentFlag=TRUE
      fi
    done
    i=$(( i+1 ))
  done
}


# workaround due to there being no sync flags to say the {APPS} are ready post WAS start
syncBySleeping()
{
  timeStart=`date +%s`
  echo "pause 5 minutes to wait for all applications to start..."
  sleep 5m
  timeEnd=`date +%s`
  timeTaken=`expr $timeEnd - $timeStart`
  echo "syncBySleeping: slept for ["$timeTaken"] Sec"
}


##################################################################################
#################
# ---- MAIN --- #
#################

scriptName=$0
if [ $# != 3 ]; then
  echo "[${scriptName}] ERROR: script "${scriptName}" has wrong input parameters"
  echo "should have 3 parameters :: RunSetup=Y/N MTM=Y/N ImportUser=Y/N"
  exit 1
fi

RunSetup=$1
MTM=$2
ImportUser=$3

echo "Run JTS Setup ["${RunSetup}"}"
echo "set up MTM ["${MTM}"}"
echo "Import LDAP Users ["${ImportUser}"}"

envProp=/etc/env.properties
touch $envProp
. $envProp

# get credentials
userCreds=$AUTO_PREP/UserParameters/userCreds.properties
. $userCreds

# Create local WAS ADMIN PASSWORD the installation
# ======================================
tempPassword=`echo ${WAS_PASSWORD} | base64 -d`

#  user may not want to run the scripting
if [ "N" == $RunSetup ]; then
  echo "[${scriptName}] User has selected to NOT run the automated scripted JTS Setup."
  echo "User will now be required to MANUALLY complete JTS Setup"
  exit 0
fi


JTSSetup_DIR=`pwd -P`
echo "" >> $envProp
echo JTSSetup_DIR=$JTSSetup_DIR>> $envProp
echo JTSSetup_DIR: [$JTSSetup_DIR]


BuildResponseFile=$JTSSetup_DIR/SetupResponse.properties


##globalTagListing=$1
##echo "globalTagListing ["${globalTagListing}"]"


###########################
# ---- processTagList --- #
#############################################################
# expect ${globalTagListing} to have already been processed #
# -- so $hosts already set                                  #
#############################################################


#################################
# -- get the LDAP properties -- #
#################################
WAS_SCRIPTS_DIR="$WAS_COMMON_DIR/WASScripts"
WAS_LDAP_DIR="$WAS_COMMON_DIR/LDAP"
JTS_DIR=/opt/IBM/JazzTeamServer/server

if [ -e "$WAS_LDAP_DIR/LDAPSecurity.properties" ]
then
  echo "sourcing $WAS_LDAP_DIR/LDAPSecurity.properties"
  . "$WAS_LDAP_DIR/LDAPSecurity.properties"
else 
  echo "[${scriptName}] ERRIR: $WAS_LDAP_DIR/LDAPSecurity.properties does not exist."
  exit 1
fi


#############################
# ---- nodes to process --- #
#############################
echo "NUMBER_OF_CUSTOM_NODES to process ["${NUMBER_OF_CUSTOM_NODES}"]"
echo "NUMBER_OF_DWA_NODES to process ["${NUMBER_OF_DWA_NODES}"]"


#########################
### BUILD RESPONSE FILE #
#########################
sh $JTSSetup_DIR/buildSetupResponse.sh
if [ $? -ne 0 ]; then
   echo "ERROR: Failed to create jts setup response file"
   exit 1
fi

# only enable MTM if required
if [ "Y" == $MTM ]; then
  # skip MTM if DM present
  isDMpresent ${NUMBER_OF_CUSTOM_NODES}
  if [ "TRUE" == ${DMpresentFlag} ]; then
    echo "WARNING: DM present so cannot Deploy MTM Sample"
  else
    echo "Deploy MTM Sample"
    sed -i s%MTM=false%MTM=true% $BuildResponseFile	
  fi
else
  echo "not deploying MTM"
fi


########################
# confirm where we are #
########################
pwd
chmod 755 *
ls -la


########################################################
# update JTS teamserver.properties file to allow access #
#########################################################
# save current teamserver.properties file and update to allow access 
cp $JTS_DIR/conf/jts/teamserver.properties $JTS_DIR/conf/jts/teamserver.properties.bak
# change com.ibm.team.repository.ws.allow.admin.access=false to true
sed -i /allow.admin.access/s%false%true% $JTS_DIR/conf/jts/teamserver.properties


###################
# Restart jts_war #
###################
$WAS_SCRIPTS_DIR/RestartSingleWAS.sh
if [ $? -ne 0 ]
then
	echo "WebSphere Server failed to restart"
	exit 1
fi


#################################
# sleep to allow {APPS}to start #
### -- not required           ###
#################################
# syncBySleeping


#################
# RUN JTS SETUP #
#################
echo "run jts setup"
cd $JTS_DIR
isDMpresent ${NUMBER_OF_CUSTOM_NODES}
if [ "TRUE" == ${DMpresentFlag} ]; then
    BuildResponseFile=$BuildResponseFile jazz_ldap_primaryid=$jazz_ldap_primaryid jazz_ldap_primaryid_Password=$tempPassword su $WAS_USER_NAME -c 'export VMARGS=-Dcom.ibm.team.repository.common.transport.TeamServerConfiguration.socketTimeout=1200000; ./repotools-jts.sh -setup includeLifecycleProjectStep=true adminUserId=$jazz_ldap_primaryid adminPassword=$jazz_ldap_primaryid_Password parametersFile=$BuildResponseFile;if [ $? -ne 0 ]
then
exit 1
fi'
else 
   BuildResponseFile=$BuildResponseFile jazz_ldap_primaryid=$jazz_ldap_primaryid jazz_ldap_primaryid_Password=$tempPassword su $WAS_USER_NAME -c './repotools-jts.sh -setup includeLifecycleProjectStep=true adminUserId=$jazz_ldap_primaryid adminPassword=$jazz_ldap_primaryid_Password parametersFile=$BuildResponseFile;if [ $? -ne 0 ]
then
exit 1
fi'
fi
# process su command response
if [ $? -ne 0 ]; then
		echo "[${scriptName}] ERROR: JTS Setup Wizard Automation failed"
		exit 1
else
		echo "------JTS Setup Wizard Completed------"
fi

# confirm it completed successfully by parsing the repotools-jts_setup.log
cat repotools-jts_setup.log | grep -i "Setup completed successfully"
if [ $? -ne 0 ]; then
		echo "[${scriptName}] ERROR: JTS Setup Wizard Automation failed"
		exit 1
else
		# might have captured an error though
		cat repotools-jts_setup.log | grep -i "error"
		if [ $? -ne 0 ]; then
  		echo "------JTS Setup Wizard Successful------"
		else
  		echo "[${scriptName}] ERROR: JTS Setup Wizard Automation failed"
  		exit 1
		fi
fi


#####################
# IMPORT LDAP USERS #
#####################
. /etc/env.properties
if [ "Y" == $ImportUser ]; then
		echo "Import All LDAP users"

		cd $JTS_DIR
		JTSURLHost=$JTSURLHost jazz_ldap_primaryid=$jazz_ldap_primaryid jazz_ldap_primaryid_Password=$tempPassword su $WAS_USER_NAME -c './repotools-jts.sh -syncUsers repositoryURL=https://$JTSURLHost:9443/jts adminUserId=$jazz_ldap_primaryid adminPassword=$jazz_ldap_primaryid_Password logFile=repotools-jts_syncUsers.log;if [ $? -ne 0 ]
		then
  	exit 1
	fi'
	# process su command response
	if [ $? -ne 0 ]; then
  	echo "[${scriptName}] ERROR: Import All LDAP users failed"
  	exit 1
	else
  	echo "LDAP users imported"
	fi
	sleep 300

else
		echo "NOT importing LDAP users"	
fi


###################
# --- the End --- #
###################

echo "-- EOF [${scriptName}] --"
exit 0
