#!/bin/bash
#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2016. All Rights Reserved.
#  
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************


#############
## FUNCTIONS 
#############


setDefaultInputs()
{
  USERNAME="defaultUSERNAME"
  USERPASSWORD="defaultUSERPASSWORD"
  UCDSERVER="defaultUCDSERVER"
  COMPONENTNAME="defaultCOMPONENTNAME"
  COMPONENTVERSION="defaultCOMPONENTVERSION"
}


checkDefaultInputs()
{
  
  if [ "$USERNAME" == "defaultUSERNAME" ]; then
    errMsg="USERNAME [$USERNAME] still contains its default value"
    exitWithErrMsg "$errMsg"
  fi
  if [ "$USERPASSWORD" == "defaultUSERPASSWORD" ]; then
    errMsg="USERPASSWORD [$USERPASSWORD] still contains its default value"
    exitWithErrMsg "$errMsg"
  fi
  if [ "$UCDSERVER" == "defaultUCDSERVER" ]; then
    errMsg="UCDSERVER [$UCDSERVER] still contains its default value"
    exitWithErrMsg "$errMsg"
  fi
  if [ "$COMPONENTNAME" == "defaultCOMPONENTNAME" ]; then
    errMsg="COMPONENTNAME [$COMPONENTNAME] still contains its default value"
    exitWithErrMsg "$errMsg"
  fi
  if [ "$COMPONENTVERSION" == "defaultCOMPONENTVERSION" ]; then
    errMsg="COMPONENTVERSION [$COMPONENTVERSION] still contains its default value"
    exitWithErrMsg "$errMsg"
  fi

}


exitWithErrMsg()
{
  tempString=$1
  echo "-- ERROR [${scriptName}] --"
  echo "-- message: $tempString"
  exit 1
}


checkFileExists()
{
  tempFile=$1
  if [ -f $tempFile ]; then
    echo "  file [$tempFile] exists"
  else
    errMsg="file [$tempFile] does not exist"
    exitWithErrMsg "$errMsg"
  fi
}


checkFolderExists()
{
  tempFolder=$1
  if [ -d $tempFolder ]; then
    echo "  folder [$tempFolder] located"
  else
    errMsg="[$tempFolder] does not exist"
    exitWithErrMsg "$errMsg"
  fi
}




# will assume these are globals
# $USERNAME
# $USERPASSWORD
# $UCDSERVER
# $UCDCLIEXE
downloadArtifacts()
{
  componentName=$1
  componentVersionName=$2
  
  echo " downloading the files from component ["${componentName}"] version ["${componentVersionName}"] to current directory $workingDir"
  COMMAND=${UCDCLIEXE}" -username "${USERNAME}" -password "${USERPASSWORD}" -weburl "${UCDSERVER}" downloadVersionArtifacts -component  "${componentName}" -version "${componentVersionName}" -location ." 
  # echo COMMAND: [${COMMAND}]
  # UCD client always exit code number 1 even with success command, so we check the output
  output=`$COMMAND`
  echo $output
  tempFile=${componentName}_${componentVersionName}_artifacts.zip
  # if [ ! -f ${componentName}_${componentVersionName}_artifacts.zip ]; then
  if [ ! -f ${tempFile} ]; then
     echo "  error: there may be more information in ${tempFile}"
     errMsg="Failed to download file, can't find the zip file [${componentName}_${componentVersionName}_artifacts.zip]"
     exitWithErrMsg "$errMsg"
  fi 
  
  echo $output | grep "Request error"  
  if [ $? != 0 ]; then
    echo "  successful"
  else
    echo "  error: there may be more information in ${tempFile}"
    errMsg="failed to download artifacts from component [${componentName}] version [${componentVersionName}]"
    exitWithErrMsg "$errMsg"
  fi
}


## usage deleteFile (force) from folder
deleteFile()
{
  tempFolder=$1
  tempFile=$2
  fileToDelete=${tempFolder}"/"${tempFile}

  if [ -f ${fileToDelete} ]; then
    echo "delete file ["${fileToDelete}"]"
    rm -f ${fileToDelete}
    if [ -f ${fileToDelete} ]; then
      errMsg="failed to delete file"
      exitWithErrMsg "${errMsg}"
    else
      echo "succesfully deleted [${fileToDelete}]"
    fi
  else
    echo "file [${fileToDelete}] does not exist, so cannot be deleted"
  fi
}


####################
##                ##
## ==== MAIN ==== ##
##                ##
####################


# UCDCLIEXE="./udclient"
# BaseDir="./unzipped"
workingDir=`pwd -P`
scriptName=$0

# -- set default values
echo "setting default inputs values"
setDefaultInputs

# -- process inputs
echo "reading inputs values"
if [ $# != 5 ]; then
  echo "ERROR: script "${scriptName}" requires 5 arguments: USERNAME=<UserName> USERPASSWORD=<UserPassword> UCDSERVER=<UCDURL ex:https://UCDHostName:8443> COMPONENTNAME=<ComponentName> COMPONENTVERSION=<ExistingComponentVersion>"
  exit 1
fi

while [[ $# > 0 ]]; do
   key=`echo $1 | cut -d'=' -f 1`
   value=`echo $1 | cut -d'=' -f 1 --complement`
   echo "[$key] [$value]"
   if [ "USERNAME"  == "$key" ]; then
      USERNAME=$value
   fi
   if [ "USERPASSWORD"  == "$key" ]; then
      USERPASSWORD=$value
   fi
   if [ "UCDSERVER"  == "$key" ]; then
      UCDSERVER=$value
   fi
   if [ "COMPONENTNAME"  == "$key" ]; then
      COMPONENTNAME=$value
   fi
   if [ "COMPONENTVERSION"  == "$key" ]; then
      COMPONENTVERSION=$value
   fi

   shift
done

# -- check all the defaults changed
echo "checking default inputs values are updated"
checkDefaultInputs

# -- ucd cli client needs JAVA_HOME set, so check it exists
echo "JAVA_HOME currently [${JAVA_HOME}]"
if [ "" == "${JAVA_HOME}" ]; then
  errMsg="You must have JAVA_HOME set on your environment to use the uDeploy client."
  exitWithErrMsg "$errMsg"
else
  echo "check [${JAVA_HOME}] folder exists"
  checkFolderExists ${JAVA_HOME}
fi

# -- check the ucd cli client exists
echo "check the ucd cli client exists"
echo "UCDCLIEXE currently [${UCDCLIEXE}]"
if [ "" == "${UCDCLIEXE}" ]; then
  errMsg="You must have the location of the ucd cli client [UCDCLIEXE] set (exported) on your environment."
  exitWithErrMsg "$errMsg"
else
  echo "check [${UCDCLIEXE}] file exists"
  checkFileExists ${UCDCLIEXE}
fi

# -- delete any old zip file
echo "delete existing scripts (as a zip) relating to component [${COMPONENTNAME}] version [${COMPONENTVERSION}] in current folder"
tempFile=${COMPONENTNAME}_${COMPONENTVERSION}_artifacts.zip
deleteFile $workingDir $tempFile

# -- download component zip file
echo "download scripts (as a zip) from component [${COMPONENTNAME}] version [${COMPONENTVERSION}] to current folder"
downloadArtifacts ${COMPONENTNAME} ${COMPONENTVERSION}

echo "SUCCESS [${scriptName}]: successfully downloaded scripts from UCD"
exit 0
