#!/bin/bash
#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2015, 2016. All Rights Reserved.
#  
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************



#############
## FUNCTIONS 
#############


setDefaultInputs()
{
  USERNAME="defaultUSERNAME"
  USERPASSWORD="defaultUSERPASSWORD"
  UCDSERVER="defaultUCDSERVER"
  APPLICATIONVERSION="defaultAPPLICATIONVERSION"
  COMPONENTVERSION="defaultCOMPONENTVERSION"
  BASEFOLDER="defaultBASEFOLDER"
}


checkDefaultInputs()
{
  if [ "$USERNAME" == "defaultUSERNAME" ]; then
    errMsg="USERNAME [$USERNAME] still contains its default value"
    exitWithErrMsg "$errMsg"
  fi
  if [ "$USERPASSWORD" == "defaultUSERPASSWORD" ]; then
    errMsg="USERPASSWORD [$USERPASSWORD] still contains its default value"
    exitWithErrMsg "$errMsg"
  fi
  if [ "$UCDSERVER" == "defaultUCDSERVER" ]; then
    errMsg="UCDSERVER [$UCDSERVER] still contains its default value"
    exitWithErrMsg "$errMsg"
  fi
  if [ "$APPLICATIONVERSION" == "defaultAPPLICATIONVERSION" ]; then
    errMsg="APPLICATIONVERSION [$APPLICATIONVERSION] still contains its default value"
    exitWithErrMsg "$errMsg"
  fi
  if [ "$COMPONENTVERSION" == "defaultCOMPONENTVERSION" ]; then
    errMsg="COMPONENTVERSION [$COMPONENTVERSION] still contains its default value"
    exitWithErrMsg "$errMsg"
  fi
  if [ "$BASEFOLDER" == "defaultBASEFOLDER" ]; then
    errMsg="BASEFOLDER [$BASEFOLDER] still contains its default value"
    exitWithErrMsg "$errMsg"
  fi
}


exitWithErrMsg()
{
  tempString=$1
  echo "-- ERROR [${scriptName}] --"
  echo "-- message: $tempString"
  exit 1
}


checkFileExists()
{
  tempFile=$1
  if [ -f $tempFile ]; then
    echo "  file [$tempFile] exists"
  else
    errMsg="file [$tempFile] does not exist"
    exitWithErrMsg "$errMsg"
  fi
}


checkFolderExists()
{
  tempFolder=$1
  if [ -d $tempFolder ]; then
    echo "  folder [$tempFolder] exists"
  else
    errMsg="folder [$tempFolder] does not exist"
    exitWithErrMsg "$errMsg"
  fi
}




####################
##                ##
## ==== MAIN ==== ##
##                ##
####################

# UCDCLIEXE="./udclient"
workingDir=`pwd -P`
scriptName=$0


# -- set default values
echo "setting default inputs values"
setDefaultInputs

# -- process inputs
echo "reading inputs values"
if [ $# != 6 ]; then
  echo "ERROR: script "${scriptName}" requires 6 arguments: USERNAME=<UserName> USERPASSWORD=<UserPassword> UCDSERVER=<UCDURL ex:https://UCDHostName:8443> BASEFOLDER=<base folder containing folders containing scripts to import> APPLICATIONVERSION=<ApplicationVersion> COMPONENTVERSION=<ComponentVersion>"
  exit 1
fi

while [[ $# > 0 ]]; do
   key=`echo $1 | cut -d'=' -f 1`
   value=`echo $1 | cut -d'=' -f 1 --complement`
   echo "[$key] [$value]"
   if [ "USERNAME"  == "$key" ]; then
      USERNAME=$value
   fi
   if [ "USERPASSWORD"  == "$key" ]; then
      USERPASSWORD=$value
   fi
   if [ "UCDSERVER"  == "$key" ]; then
      UCDSERVER=$value
   fi
   if [ "APPLICATIONVERSION"  == "$key" ]; then
      APPLICATIONVERSION=$value
   fi
   if [ "COMPONENTVERSION"  == "$key" ]; then
      COMPONENTVERSION=$value
   fi
   if [ "BASEFOLDER"  == "$key" ]; then
      BASEFOLDER=$value
   fi

   shift
done

# -- check all the defaults changed
echo "checking default inputs values are updated"
checkDefaultInputs

# -- check folder containing all the component(s) scripts exists
echo "Check ["${BASEFOLDER}"] exists"
checkFolderExists ${BASEFOLDER}

# -- ucd cli client needs JAVA_HOME set, so check it exists
echo "JAVA_HOME currently [${JAVA_HOME}]"
if [ "" == "${JAVA_HOME}" ]; then
  errMsg="You must have JAVA_HOME set on your environment to use the uDeploy client."
  exitWithErrMsg "$errMsg"
else
  echo "check [${JAVA_HOME}] folder exists"
  checkFolderExists ${JAVA_HOME}
fi

# -- check the ucd cli client exists
echo "check the ucd cli client exists"
echo "UCDCLIEXE currently [${UCDCLIEXE}]"
if [ "" == "${UCDCLIEXE}" ]; then
  errMsg="You must have the location of the ucd cli client [UCDCLIEXE] set (exported) on your environment."
  exitWithErrMsg "$errMsg"
else
  echo "check [${UCDCLIEXE}] file exists"
  checkFileExists ${UCDCLIEXE}
fi

# -- add new component version including scripts
echo "add Quick Deployer component scripts"

LIST="Application ApplicationServer Database InstallationManager SystemPre-Requisite"

for Component in $LIST
do

  echo "== ${Component}"  
  COMPONENTNAME="Rational_QD_"${Component}"_"${APPLICATIONVERSION}
  COMPONENTFOLDER=${BASEFOLDER}"/"${APPLICATIONVERSION}"/"${COMPONENTVERSION}"/"${Component}
  
  # -- check folder containing all the component(s) scripts exists
  echo "Check ["${COMPONENTFOLDER}"] exists"
  checkFolderExists ${COMPONENTFOLDER}
  
  sh ./componentVersionImport.sh USERNAME=$USERNAME USERPASSWORD=$USERPASSWORD UCDSERVER=$UCDSERVER COMPONENTNAME=$COMPONENTNAME COMPONENTVERSION=$COMPONENTVERSION BASEFOLDER=$COMPONENTFOLDER
  if [ $? == 0 ]; then
    echo "== done" 
  else
    errMsg="failed to add new component version including scripts"
    exitWithErrMsg "$errMsg"
  fi
  
done

echo "SUCCESS [${scriptName}]: Quick Deployer scripts imported into UCD"
exit 0
